@@ -1,4 +1,4 @@
-/* $XFree86: xc/lib/GL/glx/single2.c,v 1.6 2003/10/23 15:30:09 tsi Exp $ */
+/* $XFree86: xc/lib/GL/glx/single2.c,v 1.16 2004/12/10 16:06:56 alanh Exp $ */
 /*
 ** License Applicability. Except to the extent portions of this file are
 ** made subject to an alternative license as permitted in the SGI Free
@@ -35,8 +35,10 @@
 */
 
 #define NEED_GL_FUNCS_WRAPPED
+#include <stdio.h>
 #include "glxclient.h"
 #include "packsingle.h"
+#include "glxextensions.h"
 
 /* Used for GL_ARB_transpose_matrix */
 static void TransposeMatrixf(GLfloat m[16])
@@ -126,10 +128,30 @@ void glGetClipPlane(GLenum plane, GLdouble *equation)
     __GLX_SINGLE_END();
 }
 
+#define CASE_ARRAY_ENABLE(enum_name,array,dest,gl_type) \
+    case GL_ ## enum_name ## _ARRAY: \
+      *dest = (gl_type) (IS_ARRAY_ENABLED(state, array)); break
+#define CASE_ARRAY_SIZE(enum_name,array,dest,gl_type) \
+    case GL_ ## enum_name ## _ARRAY_SIZE: \
+      *dest = (gl_type) state->vertArray.arrays[array ## _ARRAY].size ; break
+#define CASE_ARRAY_TYPE(enum_name,array,dest,gl_type) \
+    case GL_ ## enum_name ## _ARRAY_TYPE: \
+      *dest = (gl_type) state->vertArray.arrays[array ## _ARRAY].type ; break
+#define CASE_ARRAY_STRIDE(enum_name,array,dest,gl_type) \
+    case GL_ ## enum_name ## _ARRAY_STRIDE: \
+      *dest = (gl_type) state->vertArray.arrays[array ## _ARRAY].stride ; break
+
+#define CASE_ARRAY_ALL(enum_name,array,dest,gl_type) \
+	CASE_ARRAY_ENABLE(enum_name,array,dest,gl_type); \
+	CASE_ARRAY_STRIDE(enum_name,array,dest,gl_type); \
+	CASE_ARRAY_TYPE(enum_name,array,dest,gl_type); \
+	CASE_ARRAY_SIZE(enum_name,array,dest,gl_type)
+
 void glGetBooleanv(GLenum val, GLboolean *b)
 {
     const GLenum origVal = val;
     __GLX_SINGLE_DECLARE_VARIABLES();
+    __GLXattribute * state = (__GLXattribute *)(gc->client_state_private);
     xGLXSingleReply reply;
 
     if (val == GL_TRANSPOSE_MODELVIEW_MATRIX_ARB) {
@@ -165,124 +187,99 @@ void glGetBooleanv(GLenum val, GLboolean *b)
 	*/
 	switch (val) {
 	  case GL_PACK_ROW_LENGTH:
-	    *b = (GLboolean)gc->state.storePack.rowLength;
+	    *b = (GLboolean)state->storePack.rowLength;
 	    break;
 	  case GL_PACK_IMAGE_HEIGHT:
-	    *b = (GLboolean)gc->state.storePack.imageHeight;
+	    *b = (GLboolean)state->storePack.imageHeight;
 	    break;
 	  case GL_PACK_SKIP_ROWS:
-	    *b = (GLboolean)gc->state.storePack.skipRows;
+	    *b = (GLboolean)state->storePack.skipRows;
 	    break;
 	  case GL_PACK_SKIP_PIXELS:
-	    *b = (GLboolean)gc->state.storePack.skipPixels;
+	    *b = (GLboolean)state->storePack.skipPixels;
 	    break;
 	  case GL_PACK_SKIP_IMAGES:
-	    *b = (GLboolean)gc->state.storePack.skipImages;
+	    *b = (GLboolean)state->storePack.skipImages;
 	    break;
 	  case GL_PACK_ALIGNMENT:
-	    *b = (GLboolean)gc->state.storePack.alignment;
+	    *b = (GLboolean)state->storePack.alignment;
 	    break;
 	  case GL_PACK_SWAP_BYTES:
-	    *b = (GLboolean)gc->state.storePack.swapEndian;
+	    *b = (GLboolean)state->storePack.swapEndian;
 	    break;
 	  case GL_PACK_LSB_FIRST:
-	    *b = (GLboolean)gc->state.storePack.lsbFirst;
+	    *b = (GLboolean)state->storePack.lsbFirst;
 	    break;
 	  case GL_UNPACK_ROW_LENGTH:
-	    *b = (GLboolean)gc->state.storeUnpack.rowLength;
+	    *b = (GLboolean)state->storeUnpack.rowLength;
 	    break;
 	  case GL_UNPACK_IMAGE_HEIGHT:
-	    *b = (GLboolean)gc->state.storeUnpack.imageHeight;
+	    *b = (GLboolean)state->storeUnpack.imageHeight;
 	    break;
 	  case GL_UNPACK_SKIP_ROWS:
-	    *b = (GLboolean)gc->state.storeUnpack.skipRows;
+	    *b = (GLboolean)state->storeUnpack.skipRows;
 	    break;
 	  case GL_UNPACK_SKIP_PIXELS:
-	    *b = (GLboolean)gc->state.storeUnpack.skipPixels;
+	    *b = (GLboolean)state->storeUnpack.skipPixels;
 	    break;
 	  case GL_UNPACK_SKIP_IMAGES:
-	    *b = (GLboolean)gc->state.storeUnpack.skipImages;
+	    *b = (GLboolean)state->storeUnpack.skipImages;
 	    break;
 	  case GL_UNPACK_ALIGNMENT:
-	    *b = (GLboolean)gc->state.storeUnpack.alignment;
+	    *b = (GLboolean)state->storeUnpack.alignment;
 	    break;
 	  case GL_UNPACK_SWAP_BYTES:
-	    *b = (GLboolean)gc->state.storeUnpack.swapEndian;
+	    *b = (GLboolean)state->storeUnpack.swapEndian;
 	    break;
 	  case GL_UNPACK_LSB_FIRST:
-	    *b = (GLboolean)gc->state.storeUnpack.lsbFirst;
-	    break;
-	  case GL_VERTEX_ARRAY:
-	    *b = (GLboolean)gc->state.vertArray.vertex.enable;
-	    break;
-	  case GL_VERTEX_ARRAY_SIZE:
-	    *b = (GLboolean)gc->state.vertArray.vertex.size;
-	    break;
-	  case GL_VERTEX_ARRAY_TYPE:
-	    *b = (GLboolean)gc->state.vertArray.vertex.type;
-	    break;
-	  case GL_VERTEX_ARRAY_STRIDE:
-	    *b = (GLboolean)gc->state.vertArray.vertex.stride;
-	    break;
-	  case GL_NORMAL_ARRAY:
-	    *b = (GLboolean)gc->state.vertArray.normal.enable;
-	    break;
-	  case GL_NORMAL_ARRAY_TYPE:
-	    *b = (GLboolean)gc->state.vertArray.normal.type;
-	    break;
-	  case GL_NORMAL_ARRAY_STRIDE:
-	    *b = (GLboolean)gc->state.vertArray.normal.stride;
-	    break;
-	  case GL_COLOR_ARRAY:
-	    *b = (GLboolean)gc->state.vertArray.color.enable;
-	    break;
-	  case GL_COLOR_ARRAY_SIZE:
-	    *b = (GLboolean)gc->state.vertArray.color.size;
-	    break;
-	  case GL_COLOR_ARRAY_TYPE:
-	    *b = (GLboolean)gc->state.vertArray.color.type;
-	    break;
-	  case GL_COLOR_ARRAY_STRIDE:
-	    *b = (GLboolean)gc->state.vertArray.color.stride;
-	    break;
-	  case GL_INDEX_ARRAY:
-	    *b = (GLboolean)gc->state.vertArray.index.enable;
-	    break;
-	  case GL_INDEX_ARRAY_TYPE:
-	    *b = (GLboolean)gc->state.vertArray.index.type;
-	    break;
-	  case GL_INDEX_ARRAY_STRIDE:
-	    *b = (GLboolean)gc->state.vertArray.index.stride;
+	    *b = (GLboolean)state->storeUnpack.lsbFirst;
 	    break;
+
+	  CASE_ARRAY_ALL(VERTEX, vertex, b, GLboolean);
+
+	  CASE_ARRAY_ENABLE(NORMAL, normal, b, GLboolean);
+	  CASE_ARRAY_TYPE(NORMAL, normal, b, GLboolean);
+	  CASE_ARRAY_STRIDE(NORMAL, normal, b, GLboolean);
+
+	  CASE_ARRAY_ALL(COLOR, color, b, GLboolean);
+
+	  CASE_ARRAY_ENABLE(INDEX, index, b, GLboolean);
+	  CASE_ARRAY_TYPE(INDEX, index, b, GLboolean);
+	  CASE_ARRAY_STRIDE(INDEX, index, b, GLboolean);
+
 	  case GL_TEXTURE_COORD_ARRAY:
-	    *b = (GLboolean)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].enable;
+	    *b = (GLboolean)IS_TEXARRAY_ENABLED(state, state->vertArray.activeTexture);
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_SIZE:
-	    *b = (GLboolean)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].size;
+	    *b = (GLboolean)state->vertArray.texCoord[state->vertArray.activeTexture].size;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_TYPE:
-	    *b = (GLboolean)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].type;
+	    *b = (GLboolean)state->vertArray.texCoord[state->vertArray.activeTexture].type;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_STRIDE:
-	    *b = (GLboolean)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].stride;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY:
-	    *b = (GLboolean)gc->state.vertArray.edgeFlag.enable;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY_STRIDE:
-	    *b = (GLboolean)gc->state.vertArray.edgeFlag.stride;
+	    *b = (GLboolean)state->vertArray.texCoord[state->vertArray.activeTexture].stride;
 	    break;
+
+	  CASE_ARRAY_ENABLE(EDGE_FLAG, edgeFlag, b, GLboolean);
+	  CASE_ARRAY_STRIDE(EDGE_FLAG, edgeFlag, b, GLboolean);
+
+	  CASE_ARRAY_ALL(SECONDARY_COLOR, secondaryColor, b, GLboolean);
+
+	  CASE_ARRAY_ENABLE(FOG_COORD, fogCoord, b, GLboolean);
+	  CASE_ARRAY_TYPE(FOG_COORD, fogCoord, b, GLboolean);
+	  CASE_ARRAY_STRIDE(FOG_COORD, fogCoord, b, GLboolean);
+
 	  case GL_MAX_ELEMENTS_VERTICES:
-	    *b = (GLboolean)gc->state.vertArray.maxElementsVertices;
+	    *b = (GLboolean)state->vertArray.maxElementsVertices;
 	    break;
 	  case GL_MAX_ELEMENTS_INDICES:
-	    *b = (GLboolean)gc->state.vertArray.maxElementsIndices;
+	    *b = (GLboolean)state->vertArray.maxElementsIndices;
 	    break;
 	  case GL_MAX_CLIENT_ATTRIB_STACK_DEPTH:
 	    *b = (GLboolean)__GL_CLIENT_ATTRIB_STACK_DEPTH;
 	    break;
 	  case GL_CLIENT_ACTIVE_TEXTURE_ARB:
-	    *b = (GLboolean)(gc->state.vertArray.activeTexture + GL_TEXTURE0_ARB);
+	    *b = (GLboolean)(state->vertArray.activeTexture + GL_TEXTURE0_ARB);
 	    break;
 	  default:
 	    /*
@@ -306,6 +303,7 @@ void glGetDoublev(GLenum val, GLdouble *d)
 {
     const GLenum origVal = val;
     __GLX_SINGLE_DECLARE_VARIABLES();
+    __GLXattribute * state = (__GLXattribute *)(gc->client_state_private);
     xGLXSingleReply reply;
 
     if (val == GL_TRANSPOSE_MODELVIEW_MATRIX_ARB) {
@@ -341,124 +339,99 @@ void glGetDoublev(GLenum val, GLdouble *d)
 	*/
 	switch (val) {
 	  case GL_PACK_ROW_LENGTH:
-	    *d = (GLdouble)gc->state.storePack.rowLength;
+	    *d = (GLdouble)state->storePack.rowLength;
 	    break;
 	  case GL_PACK_IMAGE_HEIGHT:
-	    *d = (GLdouble)gc->state.storePack.imageHeight;
+	    *d = (GLdouble)state->storePack.imageHeight;
 	    break;
 	  case GL_PACK_SKIP_ROWS:
-	    *d = (GLdouble)gc->state.storePack.skipRows;
+	    *d = (GLdouble)state->storePack.skipRows;
 	    break;
 	  case GL_PACK_SKIP_PIXELS:
-	    *d = (GLdouble)gc->state.storePack.skipPixels;
+	    *d = (GLdouble)state->storePack.skipPixels;
 	    break;
 	  case GL_PACK_SKIP_IMAGES:
-	    *d = (GLdouble)gc->state.storePack.skipImages;
+	    *d = (GLdouble)state->storePack.skipImages;
 	    break;
 	  case GL_PACK_ALIGNMENT:
-	    *d = (GLdouble)gc->state.storePack.alignment;
+	    *d = (GLdouble)state->storePack.alignment;
 	    break;
 	  case GL_PACK_SWAP_BYTES:
-	    *d = (GLdouble)gc->state.storePack.swapEndian;
+	    *d = (GLdouble)state->storePack.swapEndian;
 	    break;
 	  case GL_PACK_LSB_FIRST:
-	    *d = (GLdouble)gc->state.storePack.lsbFirst;
+	    *d = (GLdouble)state->storePack.lsbFirst;
 	    break;
 	  case GL_UNPACK_ROW_LENGTH:
-	    *d = (GLdouble)gc->state.storeUnpack.rowLength;
+	    *d = (GLdouble)state->storeUnpack.rowLength;
 	    break;
 	  case GL_UNPACK_IMAGE_HEIGHT:
-	    *d = (GLdouble)gc->state.storeUnpack.imageHeight;
+	    *d = (GLdouble)state->storeUnpack.imageHeight;
 	    break;
 	  case GL_UNPACK_SKIP_ROWS:
-	    *d = (GLdouble)gc->state.storeUnpack.skipRows;
+	    *d = (GLdouble)state->storeUnpack.skipRows;
 	    break;
 	  case GL_UNPACK_SKIP_PIXELS:
-	    *d = (GLdouble)gc->state.storeUnpack.skipPixels;
+	    *d = (GLdouble)state->storeUnpack.skipPixels;
 	    break;
 	  case GL_UNPACK_SKIP_IMAGES:
-	    *d = (GLdouble)gc->state.storeUnpack.skipImages;
+	    *d = (GLdouble)state->storeUnpack.skipImages;
 	    break;
 	  case GL_UNPACK_ALIGNMENT:
-	    *d = (GLdouble)gc->state.storeUnpack.alignment;
+	    *d = (GLdouble)state->storeUnpack.alignment;
 	    break;
 	  case GL_UNPACK_SWAP_BYTES:
-	    *d = (GLdouble)gc->state.storeUnpack.swapEndian;
+	    *d = (GLdouble)state->storeUnpack.swapEndian;
 	    break;
 	  case GL_UNPACK_LSB_FIRST:
-	    *d = (GLdouble)gc->state.storeUnpack.lsbFirst;
-	    break;
-	  case GL_VERTEX_ARRAY:
-	    *d = (GLdouble)gc->state.vertArray.vertex.enable;
-	    break;
-	  case GL_VERTEX_ARRAY_SIZE:
-	    *d = (GLdouble)gc->state.vertArray.vertex.size;
-	    break;
-	  case GL_VERTEX_ARRAY_TYPE:
-	    *d = (GLdouble)gc->state.vertArray.vertex.type;
-	    break;
-	  case GL_VERTEX_ARRAY_STRIDE:
-	    *d = (GLdouble)gc->state.vertArray.vertex.stride;
-	    break;
-	  case GL_NORMAL_ARRAY:
-	    *d = (GLdouble)gc->state.vertArray.normal.enable;
-	    break;
-	  case GL_NORMAL_ARRAY_TYPE:
-	    *d = (GLdouble)gc->state.vertArray.normal.type;
-	    break;
-	  case GL_NORMAL_ARRAY_STRIDE:
-	    *d = (GLdouble)gc->state.vertArray.normal.stride;
-	    break;
-	  case GL_COLOR_ARRAY:
-	    *d = (GLdouble)gc->state.vertArray.color.enable;
-	    break;
-	  case GL_COLOR_ARRAY_SIZE:
-	    *d = (GLdouble)gc->state.vertArray.color.size;
-	    break;
-	  case GL_COLOR_ARRAY_TYPE:
-	    *d = (GLdouble)gc->state.vertArray.color.type;
-	    break;
-	  case GL_COLOR_ARRAY_STRIDE:
-	    *d = (GLdouble)gc->state.vertArray.color.stride;
-	    break;
-	  case GL_INDEX_ARRAY:
-	    *d = (GLdouble)gc->state.vertArray.index.enable;
-	    break;
-	  case GL_INDEX_ARRAY_TYPE:
-	    *d = (GLdouble)gc->state.vertArray.index.type;
-	    break;
-	  case GL_INDEX_ARRAY_STRIDE:
-	    *d = (GLdouble)gc->state.vertArray.index.stride;
+	    *d = (GLdouble)state->storeUnpack.lsbFirst;
 	    break;
+
+	  CASE_ARRAY_ALL(VERTEX, vertex, d, GLdouble);
+
+	  CASE_ARRAY_ENABLE(NORMAL, normal, d, GLdouble);
+	  CASE_ARRAY_TYPE(NORMAL, normal, d, GLdouble);
+	  CASE_ARRAY_STRIDE(NORMAL, normal, d, GLdouble);
+
+	  CASE_ARRAY_ALL(COLOR, color, d, GLdouble);
+
+	  CASE_ARRAY_ENABLE(INDEX, index, d, GLdouble);
+	  CASE_ARRAY_TYPE(INDEX, index, d, GLdouble);
+	  CASE_ARRAY_STRIDE(INDEX, index, d, GLdouble);
+
 	  case GL_TEXTURE_COORD_ARRAY:
-	    *d = (GLdouble)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].enable;
+	    *d = (GLdouble) IS_TEXARRAY_ENABLED(state, state->vertArray.activeTexture);
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_SIZE:
-	    *d = (GLdouble)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].size;
+	    *d = (GLdouble)state->vertArray.texCoord[state->vertArray.activeTexture].size;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_TYPE:
-	    *d = (GLdouble)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].type;
+	    *d = (GLdouble)state->vertArray.texCoord[state->vertArray.activeTexture].type;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_STRIDE:
-	    *d = (GLdouble)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].stride;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY:
-	    *d = (GLdouble)gc->state.vertArray.edgeFlag.enable;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY_STRIDE:
-	    *d = (GLdouble)gc->state.vertArray.edgeFlag.stride;
+	    *d = (GLdouble)state->vertArray.texCoord[state->vertArray.activeTexture].stride;
 	    break;
+
+	  CASE_ARRAY_ENABLE(EDGE_FLAG, edgeFlag, d, GLdouble);
+	  CASE_ARRAY_STRIDE(EDGE_FLAG, edgeFlag, d, GLdouble);
+
+	  CASE_ARRAY_ALL(SECONDARY_COLOR, secondaryColor, d, GLdouble);
+
+	  CASE_ARRAY_ENABLE(FOG_COORD, fogCoord, d, GLdouble);
+	  CASE_ARRAY_TYPE(FOG_COORD, fogCoord, d, GLdouble);
+	  CASE_ARRAY_STRIDE(FOG_COORD, fogCoord, d, GLdouble);
+
 	  case GL_MAX_ELEMENTS_VERTICES:
-	    *d = (GLdouble)gc->state.vertArray.maxElementsVertices;
+	    *d = (GLdouble)state->vertArray.maxElementsVertices;
 	    break;
 	  case GL_MAX_ELEMENTS_INDICES:
-	    *d = (GLdouble)gc->state.vertArray.maxElementsIndices;
+	    *d = (GLdouble)state->vertArray.maxElementsIndices;
 	    break;
 	  case GL_MAX_CLIENT_ATTRIB_STACK_DEPTH:
 	    *d = (GLdouble)__GL_CLIENT_ATTRIB_STACK_DEPTH;
 	    break;
 	  case GL_CLIENT_ACTIVE_TEXTURE_ARB:
-	    *d = (GLdouble)(gc->state.vertArray.activeTexture + GL_TEXTURE0_ARB);
+	    *d = (GLdouble)(state->vertArray.activeTexture + GL_TEXTURE0_ARB);
 	    break;
 	  default:
 	    /*
@@ -482,6 +455,7 @@ void glGetFloatv(GLenum val, GLfloat *f)
 {
     const GLenum origVal = val;
     __GLX_SINGLE_DECLARE_VARIABLES();
+    __GLXattribute * state = (__GLXattribute *)(gc->client_state_private);
     xGLXSingleReply reply;
 
     if (val == GL_TRANSPOSE_MODELVIEW_MATRIX_ARB) {
@@ -517,124 +491,99 @@ void glGetFloatv(GLenum val, GLfloat *f)
 	*/
 	switch (val) {
 	  case GL_PACK_ROW_LENGTH:
-	    *f = (GLfloat)gc->state.storePack.rowLength;
+	    *f = (GLfloat)state->storePack.rowLength;
 	    break;
 	  case GL_PACK_IMAGE_HEIGHT:
-	    *f = (GLfloat)gc->state.storePack.imageHeight;
+	    *f = (GLfloat)state->storePack.imageHeight;
 	    break;
 	  case GL_PACK_SKIP_ROWS:
-	    *f = (GLfloat)gc->state.storePack.skipRows;
+	    *f = (GLfloat)state->storePack.skipRows;
 	    break;
 	  case GL_PACK_SKIP_PIXELS:
-	    *f = (GLfloat)gc->state.storePack.skipPixels;
+	    *f = (GLfloat)state->storePack.skipPixels;
 	    break;
 	  case GL_PACK_SKIP_IMAGES:
-	    *f = (GLfloat)gc->state.storePack.skipImages;
+	    *f = (GLfloat)state->storePack.skipImages;
 	    break;
 	  case GL_PACK_ALIGNMENT:
-	    *f = (GLfloat)gc->state.storePack.alignment;
+	    *f = (GLfloat)state->storePack.alignment;
 	    break;
 	  case GL_PACK_SWAP_BYTES:
-	    *f = (GLfloat)gc->state.storePack.swapEndian;
+	    *f = (GLfloat)state->storePack.swapEndian;
 	    break;
 	  case GL_PACK_LSB_FIRST:
-	    *f = (GLfloat)gc->state.storePack.lsbFirst;
+	    *f = (GLfloat)state->storePack.lsbFirst;
 	    break;
 	  case GL_UNPACK_ROW_LENGTH:
-	    *f = (GLfloat)gc->state.storeUnpack.rowLength;
+	    *f = (GLfloat)state->storeUnpack.rowLength;
 	    break;
 	  case GL_UNPACK_IMAGE_HEIGHT:
-	    *f = (GLfloat)gc->state.storeUnpack.imageHeight;
+	    *f = (GLfloat)state->storeUnpack.imageHeight;
 	    break;
 	  case GL_UNPACK_SKIP_ROWS:
-	    *f = (GLfloat)gc->state.storeUnpack.skipRows;
+	    *f = (GLfloat)state->storeUnpack.skipRows;
 	    break;
 	  case GL_UNPACK_SKIP_PIXELS:
-	    *f = (GLfloat)gc->state.storeUnpack.skipPixels;
+	    *f = (GLfloat)state->storeUnpack.skipPixels;
 	    break;
 	  case GL_UNPACK_SKIP_IMAGES:
-	    *f = (GLfloat)gc->state.storeUnpack.skipImages;
+	    *f = (GLfloat)state->storeUnpack.skipImages;
 	    break;
 	  case GL_UNPACK_ALIGNMENT:
-	    *f = (GLfloat)gc->state.storeUnpack.alignment;
+	    *f = (GLfloat)state->storeUnpack.alignment;
 	    break;
 	  case GL_UNPACK_SWAP_BYTES:
-	    *f = (GLfloat)gc->state.storeUnpack.swapEndian;
+	    *f = (GLfloat)state->storeUnpack.swapEndian;
 	    break;
 	  case GL_UNPACK_LSB_FIRST:
-	    *f = (GLfloat)gc->state.storeUnpack.lsbFirst;
-	    break;
-	  case GL_VERTEX_ARRAY:
-	    *f = (GLfloat)gc->state.vertArray.vertex.enable;
-	    break;
-	  case GL_VERTEX_ARRAY_SIZE:
-	    *f = (GLfloat)gc->state.vertArray.vertex.size;
-	    break;
-	  case GL_VERTEX_ARRAY_TYPE:
-	    *f = (GLfloat)gc->state.vertArray.vertex.type;
-	    break;
-	  case GL_VERTEX_ARRAY_STRIDE:
-	    *f = (GLfloat)gc->state.vertArray.vertex.stride;
-	    break;
-	  case GL_NORMAL_ARRAY:
-	    *f = (GLfloat)gc->state.vertArray.normal.enable;
-	    break;
-	  case GL_NORMAL_ARRAY_TYPE:
-	    *f = (GLfloat)gc->state.vertArray.normal.type;
-	    break;
-	  case GL_NORMAL_ARRAY_STRIDE:
-	    *f = (GLfloat)gc->state.vertArray.normal.stride;
-	    break;
-	  case GL_COLOR_ARRAY:
-	    *f = (GLfloat)gc->state.vertArray.color.enable;
-	    break;
-	  case GL_COLOR_ARRAY_SIZE:
-	    *f = (GLfloat)gc->state.vertArray.color.size;
-	    break;
-	  case GL_COLOR_ARRAY_TYPE:
-	    *f = (GLfloat)gc->state.vertArray.color.type;
-	    break;
-	  case GL_COLOR_ARRAY_STRIDE:
-	    *f = (GLfloat)gc->state.vertArray.color.stride;
-	    break;
-	  case GL_INDEX_ARRAY:
-	    *f = (GLfloat)gc->state.vertArray.index.enable;
-	    break;
-	  case GL_INDEX_ARRAY_TYPE:
-	    *f = (GLfloat)gc->state.vertArray.index.type;
-	    break;
-	  case GL_INDEX_ARRAY_STRIDE:
-	    *f = (GLfloat)gc->state.vertArray.index.stride;
+	    *f = (GLfloat)state->storeUnpack.lsbFirst;
 	    break;
+
+	  CASE_ARRAY_ALL(VERTEX, vertex, f, GLfloat);
+
+	  CASE_ARRAY_ENABLE(NORMAL, normal, f, GLfloat);
+	  CASE_ARRAY_TYPE(NORMAL, normal, f, GLfloat);
+	  CASE_ARRAY_STRIDE(NORMAL, normal, f, GLfloat);
+
+	  CASE_ARRAY_ALL(COLOR, color, f, GLfloat);
+
+	  CASE_ARRAY_ENABLE(INDEX, index, f, GLfloat);
+	  CASE_ARRAY_TYPE(INDEX, index, f, GLfloat);
+	  CASE_ARRAY_STRIDE(INDEX, index, f, GLfloat);
+
 	  case GL_TEXTURE_COORD_ARRAY:
-	    *f = (GLfloat)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].enable;
+	    *f = (GLfloat) IS_TEXARRAY_ENABLED(state, state->vertArray.activeTexture);
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_SIZE:
-	    *f = (GLfloat)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].size;
+	    *f = (GLfloat)state->vertArray.texCoord[state->vertArray.activeTexture].size;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_TYPE:
-	    *f = (GLfloat)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].type;
+	    *f = (GLfloat)state->vertArray.texCoord[state->vertArray.activeTexture].type;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_STRIDE:
-	    *f = (GLfloat)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].stride;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY:
-	    *f = (GLfloat)gc->state.vertArray.edgeFlag.enable;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY_STRIDE:
-	    *f = (GLfloat)gc->state.vertArray.edgeFlag.stride;
+	    *f = (GLfloat)state->vertArray.texCoord[state->vertArray.activeTexture].stride;
 	    break;
+
+	  CASE_ARRAY_ENABLE(EDGE_FLAG, edgeFlag, f, GLfloat);
+	  CASE_ARRAY_STRIDE(EDGE_FLAG, edgeFlag, f, GLfloat);
+
+	  CASE_ARRAY_ALL(SECONDARY_COLOR, secondaryColor, f, GLfloat);
+
+	  CASE_ARRAY_ENABLE(FOG_COORD, fogCoord, f, GLfloat);
+	  CASE_ARRAY_TYPE(FOG_COORD, fogCoord, f, GLfloat);
+	  CASE_ARRAY_STRIDE(FOG_COORD, fogCoord, f, GLfloat);
+
 	  case GL_MAX_ELEMENTS_VERTICES:
-	    *f = (GLfloat)gc->state.vertArray.maxElementsVertices;
+	    *f = (GLfloat)state->vertArray.maxElementsVertices;
 	    break;
 	  case GL_MAX_ELEMENTS_INDICES:
-	    *f = (GLfloat)gc->state.vertArray.maxElementsIndices;
+	    *f = (GLfloat)state->vertArray.maxElementsIndices;
 	    break;
 	  case GL_MAX_CLIENT_ATTRIB_STACK_DEPTH:
 	    *f = (GLfloat)__GL_CLIENT_ATTRIB_STACK_DEPTH;
 	    break;
 	  case GL_CLIENT_ACTIVE_TEXTURE_ARB:
-	    *f = (GLfloat)(gc->state.vertArray.activeTexture + GL_TEXTURE0_ARB);
+	    *f = (GLfloat)(state->vertArray.activeTexture + GL_TEXTURE0_ARB);
 	    break;
 	  default:
 	    /*
@@ -658,6 +607,7 @@ void glGetIntegerv(GLenum val, GLint *i)
 {
     const GLenum origVal = val;
     __GLX_SINGLE_DECLARE_VARIABLES();
+    __GLXattribute * state = (__GLXattribute *)(gc->client_state_private);
     xGLXSingleReply reply;
 
     if (val == GL_TRANSPOSE_MODELVIEW_MATRIX_ARB) {
@@ -693,124 +643,99 @@ void glGetIntegerv(GLenum val, GLint *i)
 	*/
 	switch (val) {
 	  case GL_PACK_ROW_LENGTH:
-	    *i = (GLint)gc->state.storePack.rowLength;
+	    *i = (GLint)state->storePack.rowLength;
 	    break;
 	  case GL_PACK_IMAGE_HEIGHT:
-	    *i = (GLint)gc->state.storePack.imageHeight;
+	    *i = (GLint)state->storePack.imageHeight;
 	    break;
 	  case GL_PACK_SKIP_ROWS:
-	    *i = (GLint)gc->state.storePack.skipRows;
+	    *i = (GLint)state->storePack.skipRows;
 	    break;
 	  case GL_PACK_SKIP_PIXELS:
-	    *i = (GLint)gc->state.storePack.skipPixels;
+	    *i = (GLint)state->storePack.skipPixels;
 	    break;
 	  case GL_PACK_SKIP_IMAGES:
-	    *i = (GLint)gc->state.storePack.skipImages;
+	    *i = (GLint)state->storePack.skipImages;
 	    break;
 	  case GL_PACK_ALIGNMENT:
-	    *i = (GLint)gc->state.storePack.alignment;
+	    *i = (GLint)state->storePack.alignment;
 	    break;
 	  case GL_PACK_SWAP_BYTES:
-	    *i = (GLint)gc->state.storePack.swapEndian;
+	    *i = (GLint)state->storePack.swapEndian;
 	    break;
 	  case GL_PACK_LSB_FIRST:
-	    *i = (GLint)gc->state.storePack.lsbFirst;
+	    *i = (GLint)state->storePack.lsbFirst;
 	    break;
 	  case GL_UNPACK_ROW_LENGTH:
-	    *i = (GLint)gc->state.storeUnpack.rowLength;
+	    *i = (GLint)state->storeUnpack.rowLength;
 	    break;
 	  case GL_UNPACK_IMAGE_HEIGHT:
-	    *i = (GLint)gc->state.storeUnpack.imageHeight;
+	    *i = (GLint)state->storeUnpack.imageHeight;
 	    break;
 	  case GL_UNPACK_SKIP_ROWS:
-	    *i = (GLint)gc->state.storeUnpack.skipRows;
+	    *i = (GLint)state->storeUnpack.skipRows;
 	    break;
 	  case GL_UNPACK_SKIP_PIXELS:
-	    *i = (GLint)gc->state.storeUnpack.skipPixels;
+	    *i = (GLint)state->storeUnpack.skipPixels;
 	    break;
 	  case GL_UNPACK_SKIP_IMAGES:
-	    *i = (GLint)gc->state.storeUnpack.skipImages;
+	    *i = (GLint)state->storeUnpack.skipImages;
 	    break;
 	  case GL_UNPACK_ALIGNMENT:
-	    *i = (GLint)gc->state.storeUnpack.alignment;
+	    *i = (GLint)state->storeUnpack.alignment;
 	    break;
 	  case GL_UNPACK_SWAP_BYTES:
-	    *i = (GLint)gc->state.storeUnpack.swapEndian;
+	    *i = (GLint)state->storeUnpack.swapEndian;
 	    break;
 	  case GL_UNPACK_LSB_FIRST:
-	    *i = (GLint)gc->state.storeUnpack.lsbFirst;
-	    break;
-	  case GL_VERTEX_ARRAY:
-	    *i = (GLint)gc->state.vertArray.vertex.enable;
-	    break;
-	  case GL_VERTEX_ARRAY_SIZE:
-	    *i = (GLint)gc->state.vertArray.vertex.size;
-	    break;
-	  case GL_VERTEX_ARRAY_TYPE:
-	    *i = (GLint)gc->state.vertArray.vertex.type;
-	    break;
-	  case GL_VERTEX_ARRAY_STRIDE:
-	    *i = (GLint)gc->state.vertArray.vertex.stride;
-	    break;
-	  case GL_NORMAL_ARRAY:
-	    *i = (GLint)gc->state.vertArray.normal.enable;
-	    break;
-	  case GL_NORMAL_ARRAY_TYPE:
-	    *i = (GLint)gc->state.vertArray.normal.type;
-	    break;
-	  case GL_NORMAL_ARRAY_STRIDE:
-	    *i = (GLint)gc->state.vertArray.normal.stride;
-	    break;
-	  case GL_COLOR_ARRAY:
-	    *i = (GLint)gc->state.vertArray.color.enable;
-	    break;
-	  case GL_COLOR_ARRAY_SIZE:
-	    *i = (GLint)gc->state.vertArray.color.size;
-	    break;
-	  case GL_COLOR_ARRAY_TYPE:
-	    *i = (GLint)gc->state.vertArray.color.type;
-	    break;
-	  case GL_COLOR_ARRAY_STRIDE:
-	    *i = (GLint)gc->state.vertArray.color.stride;
-	    break;
-	  case GL_INDEX_ARRAY:
-	    *i = (GLint)gc->state.vertArray.index.enable;
-	    break;
-	  case GL_INDEX_ARRAY_TYPE:
-	    *i = (GLint)gc->state.vertArray.index.type;
-	    break;
-	  case GL_INDEX_ARRAY_STRIDE:
-	    *i = (GLint)gc->state.vertArray.index.stride;
+	    *i = (GLint)state->storeUnpack.lsbFirst;
 	    break;
+
+	  CASE_ARRAY_ALL(VERTEX, vertex, i, GLint);
+
+	  CASE_ARRAY_ENABLE(NORMAL, normal, i, GLint);
+	  CASE_ARRAY_TYPE(NORMAL, normal, i, GLint);
+	  CASE_ARRAY_STRIDE(NORMAL, normal, i, GLint);
+
+	  CASE_ARRAY_ALL(COLOR, color, i, GLint);
+
+	  CASE_ARRAY_ENABLE(INDEX, index, i, GLint);
+	  CASE_ARRAY_TYPE(INDEX, index, i, GLint);
+	  CASE_ARRAY_STRIDE(INDEX, index, i, GLint);
+
 	  case GL_TEXTURE_COORD_ARRAY:
-	    *i = (GLint)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].enable;
+	    *i = (GLint) IS_TEXARRAY_ENABLED(state, state->vertArray.activeTexture);
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_SIZE:
-	    *i = (GLint)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].size;
+	    *i = (GLint)state->vertArray.texCoord[state->vertArray.activeTexture].size;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_TYPE:
-	    *i = (GLint)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].type;
+	    *i = (GLint)state->vertArray.texCoord[state->vertArray.activeTexture].type;
 	    break;
 	  case GL_TEXTURE_COORD_ARRAY_STRIDE:
-	    *i = (GLint)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].stride;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY:
-	    *i = (GLint)gc->state.vertArray.edgeFlag.enable;
-	    break;
-	  case GL_EDGE_FLAG_ARRAY_STRIDE:
-	    *i = (GLint)gc->state.vertArray.edgeFlag.stride;
+	    *i = (GLint)state->vertArray.texCoord[state->vertArray.activeTexture].stride;
 	    break;
+
+	  CASE_ARRAY_ENABLE(EDGE_FLAG, edgeFlag, i, GLint);
+	  CASE_ARRAY_STRIDE(EDGE_FLAG, edgeFlag, i, GLint);
+
+	  CASE_ARRAY_ALL(SECONDARY_COLOR, secondaryColor, i, GLint);
+
+	  CASE_ARRAY_ENABLE(FOG_COORD, fogCoord, i, GLint);
+	  CASE_ARRAY_TYPE(FOG_COORD, fogCoord, i, GLint);
+	  CASE_ARRAY_STRIDE(FOG_COORD, fogCoord, i, GLint);
+
 	  case GL_MAX_ELEMENTS_VERTICES:
-	    *i = (GLint)gc->state.vertArray.maxElementsVertices;
+	    *i = (GLint)state->vertArray.maxElementsVertices;
 	    break;
 	  case GL_MAX_ELEMENTS_INDICES:
-	    *i = (GLint)gc->state.vertArray.maxElementsIndices;
+	    *i = (GLint)state->vertArray.maxElementsIndices;
 	    break;
 	  case GL_MAX_CLIENT_ATTRIB_STACK_DEPTH:
 	    *i = (GLint)__GL_CLIENT_ATTRIB_STACK_DEPTH;
 	    break;
 	  case GL_CLIENT_ACTIVE_TEXTURE_ARB:
-	    *i = (GLint)(gc->state.vertArray.activeTexture + GL_TEXTURE0_ARB);
+	    *i = (GLint)(state->vertArray.activeTexture + GL_TEXTURE0_ARB);
 	    break;
 	  default:
 	    /*
@@ -924,10 +849,29 @@ void glFinish(void)
     __GLX_SINGLE_END();
 }
 
+
+/**
+ * Extract the major and minor version numbers from a version string.
+ */
+static void
+version_from_string( const char * ver, 
+		     int * major_version, int * minor_version )
+{
+    const char * end;
+    long major;
+    long minor;
+
+    major = strtol( ver, (char **) & end, 10 );
+    minor = strtol( end + 1, NULL, 10 );
+    *major_version = major;
+    *minor_version = minor;
+}
+
+
 const GLubyte *glGetString(GLenum name)
 {
-    __GLX_SINGLE_DECLARE_VARIABLES();
-    xGLXSingleReply reply;
+    __GLXcontext *gc = __glXGetCurrentContext();
+    Display *dpy = gc->currentDpy;
     GLubyte *s = NULL;
 
     if (!dpy) return 0;
@@ -956,48 +900,123 @@ const GLubyte *glGetString(GLenum name)
     /*
     ** Get requested string from server
     */
-    __GLX_SINGLE_LOAD_VARIABLES();
-    __GLX_SINGLE_BEGIN(X_GLsop_GetString,4);
-    __GLX_SINGLE_PUT_LONG(0,name);
-    __GLX_SINGLE_READ_XREPLY();
-    __GLX_SINGLE_GET_SIZE(compsize);
-    s = (GLubyte*) Xmalloc(compsize);
+
+    (void) __glXFlushRenderBuffer( gc, gc->pc );
+    s = (GLubyte *) __glXGetStringFromServer( dpy, gc->majorOpcode,
+				  X_GLsop_GetString, gc->currentContextTag,
+				  name );
     if (!s) {
 	/* Throw data on the floor */
-	_XEatData(dpy, compsize);
 	__glXSetError(gc, GL_OUT_OF_MEMORY);
     } else {
-	__GLX_SINGLE_GET_CHAR_ARRAY(s,compsize);
-
 	/*
 	** Update local cache
 	*/
 	switch(name) {
-	  case GL_VENDOR:
+	case GL_VENDOR:
 	    gc->vendor = s;
 	    break;
-	  case GL_RENDERER:
+
+	case GL_RENDERER:
 	    gc->renderer = s;
 	    break;
-	  case GL_VERSION:
-	    gc->version = s;
+
+	case GL_VERSION: {
+	    int client_major;
+	    int client_minor;
+
+	    version_from_string( (char *) s, 
+				 & gc->server_major, & gc->server_minor );
+	    __glXGetGLVersion( & client_major, & client_minor );
+
+	    if ( (gc->server_major < client_major)
+		 || ((gc->server_major == client_major) 
+		     && (gc->server_minor <= client_minor)) ) {
+		gc->version = s;
+	    }
+	    else {
+		/* Allow 7 bytes for the client-side GL version.  This allows
+		 * for upto version 999.999.  I'm not holding my breath for
+		 * that one!  The extra 4 is for the ' ()\0' that will be
+		 * added.
+		 */
+		const size_t size = 7 + strlen( (char *) s ) + 4;
+
+		gc->version = Xmalloc( size );
+		if ( gc->version == NULL ) {
+		    /* If we couldn't allocate memory for the new string,
+		     * make a best-effort and just copy the client-side version
+		     * to the string and use that.  It probably doesn't
+		     * matter what is done here.  If there not memory available
+		     * for a short string, the system is probably going to die
+		     * soon anyway.
+		     */
+		    snprintf( (char *) s, strlen( (char *) s ) + 1, "%u.%u",
+			      client_major, client_minor );
+		    gc->version = s;
+		}
+		else {
+		    snprintf( (char *)gc->version, size, "%u.%u (%s)",
+			      client_major, client_minor, s );
+		    Xfree( s );
+		    s = gc->version;
+		}
+	    }
 	    break;
-	  case GL_EXTENSIONS:
-	    gc->extensions =
-	      (GLubyte *)__glXCombineExtensionStrings( (char *)s,
-			 __glXGLClientExtensions );
+	}
+
+	case GL_EXTENSIONS: {
+	    int major = 1;
+	    int minor = 0;
+
+	    /* This code is currently disabled.  I was reminded that some
+	     * vendors intentionally exclude some extensions from their
+	     * extension string that are part of the core version they
+	     * advertise.  In particular, on Nvidia drivers this means that
+	     * the functionality is supported by the driver, but is not
+	     * hardware accelerated.  For example, a TNT will show core
+	     * version 1.5, but most of the post-1.2 functionality is a
+	     * software fallback.
+	     * 
+	     * I don't want to break applications that rely on this odd
+	     * behavior.  At the same time, the code is written and tested,
+	     * so I didn't want to throw it away.  Therefore, the code is here
+	     * but disabled.  In the future, we may wish to and an environment
+	     * variable to enable it.
+	     */
+	    
+#if 0
+	    /* Call glGetString just to make sure that gc->server_major and
+	     * gc->server_minor are set.  This version may be higher than we
+	     * can completely support, but it may imply support for some
+	     * extensions that we can support.
+	     * 
+	     * For example, at the time of this writing, the client-side
+	     * library only supports upto core GL version 1.2.  However, cubic
+	     * textures, multitexture, multisampling, and some other 1.3
+	     * features are supported.  If the server reports back version
+	     * 1.3, but does not report all of those extensions, we will
+	     * enable them.
+	     */
+	    (void *) glGetString( GL_VERSION );
+	    major = gc->server_major,
+	    minor = gc->server_minor;
+#endif
+
+	    __glXCalculateUsableGLExtensions( gc, (char *) s, major, minor );
 	    XFree( s );
 	    s = gc->extensions;
 	    break;
 	}
+	}
     }
-    __GLX_SINGLE_END();
     return s;
 }
 
 GLboolean glIsEnabled(GLenum cap)
 {
     __GLX_SINGLE_DECLARE_VARIABLES();
+    __GLXattribute * state = (__GLXattribute *)(gc->client_state_private);
     xGLXSingleReply reply;
     GLboolean retval = 0;
 
@@ -1005,17 +1024,21 @@ GLboolean glIsEnabled(GLenum cap)
 
     switch(cap) {
       case GL_VERTEX_ARRAY:
-	  return gc->state.vertArray.vertex.enable;
+	  return IS_ARRAY_ENABLED(state, vertex);
       case GL_NORMAL_ARRAY:
-	  return gc->state.vertArray.normal.enable;
+	  return IS_ARRAY_ENABLED(state, normal);
       case GL_COLOR_ARRAY:
-	  return gc->state.vertArray.color.enable;
+	  return IS_ARRAY_ENABLED(state, color);
       case GL_INDEX_ARRAY:
-	  return gc->state.vertArray.index.enable;
+	  return IS_ARRAY_ENABLED(state, index);
       case GL_TEXTURE_COORD_ARRAY:
-	  return gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].enable;
+	  return IS_TEXARRAY_ENABLED(state, state->vertArray.activeTexture);
       case GL_EDGE_FLAG_ARRAY:
-	  return gc->state.vertArray.edgeFlag.enable;
+	  return IS_ARRAY_ENABLED(state, edgeFlag);
+      case GL_SECONDARY_COLOR_ARRAY:
+	  return IS_ARRAY_ENABLED(state, secondaryColor);
+      case GL_FOG_COORD_ARRAY:
+	  return IS_ARRAY_ENABLED(state, fogCoord);
     }
 
     __GLX_SINGLE_LOAD_VARIABLES();
@@ -1030,28 +1053,35 @@ GLboolean glIsEnabled(GLenum cap)
 void glGetPointerv(GLenum pname, void **params)
 {
     __GLXcontext *gc = __glXGetCurrentContext();
+    __GLXattribute * state = (__GLXattribute *)(gc->client_state_private);
     Display *dpy = gc->currentDpy;
 
     if (!dpy) return;
 
     switch(pname) {
       case GL_VERTEX_ARRAY_POINTER:
-	  *params = (void *)gc->state.vertArray.vertex.ptr;
+	  *params = (void *)state->vertArray.arrays[ vertex_ARRAY ].ptr;
 	  return;
       case GL_NORMAL_ARRAY_POINTER:
-	  *params = (void *)gc->state.vertArray.normal.ptr;
+	  *params = (void *)state->vertArray.arrays[ normal_ARRAY ].ptr;
 	  return;
       case GL_COLOR_ARRAY_POINTER:
-	  *params = (void *)gc->state.vertArray.color.ptr;
+	  *params = (void *)state->vertArray.arrays[ color_ARRAY ].ptr;
 	  return;
       case GL_INDEX_ARRAY_POINTER:
-	  *params = (void *)gc->state.vertArray.index.ptr;
+	  *params = (void *)state->vertArray.arrays[ index_ARRAY ].ptr;
 	  return;
       case GL_TEXTURE_COORD_ARRAY_POINTER:
-	  *params = (void *)gc->state.vertArray.texCoord[gc->state.vertArray.activeTexture].ptr;
+	  *params = (void *)state->vertArray.texCoord[state->vertArray.activeTexture].ptr;
 	  return;
       case GL_EDGE_FLAG_ARRAY_POINTER:
-	  *params = (void *)gc->state.vertArray.edgeFlag.ptr;
+	  *params = (void *)state->vertArray.arrays[ edgeFlag_ARRAY ].ptr;
+	return;
+      case GL_SECONDARY_COLOR_ARRAY_POINTER:
+	  *params = (void *)state->vertArray.arrays[ secondaryColor_ARRAY ].ptr;
+	return;
+      case GL_FOG_COORD_ARRAY_POINTER:
+	  *params = (void *)state->vertArray.arrays[ fogCoord_ARRAY ].ptr;
 	return;
       case GL_FEEDBACK_BUFFER_POINTER:
 	*params = (void *)gc->feedbackBuf;
