@@ -1,4 +1,4 @@
-/* $XFree86: xc/lib/Xxf86dga/XF86DGA2.c,v 1.24 2003/11/21 05:07:16 dawes Exp $ */
+/* $XFree86: xc/lib/Xxf86dga/XF86DGA2.c,v 1.28 2004/12/31 03:30:39 tsi Exp $ */
 /*
 
 Copyright (c) 1995  Jon Tombs
@@ -23,6 +23,9 @@ Copyright (c) 1995,1996  The XFree86 Project, Inc
 #include <X11/extensions/Xext.h>
 #include <X11/extensions/extutil.h>
 #include <stdio.h>
+#ifdef linux
+#include <sys/ioctl.h>
+#endif
 
 
 /* If you change this, change the Bases[] array below as well */
@@ -33,10 +36,16 @@ char *xdga_extension_name = XF86DGANAME;
 static XExtensionInfo _xdga_info_data;
 static XExtensionInfo *xdga_info = &_xdga_info_data;
 
+#if !defined(_LP64) && defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64)
+typedef unsigned long long mmapOffset;
+#else
+typedef unsigned long      mmapOffset;
+#endif
  
-Bool XDGAMapFramebuffer(int, char *, unsigned char*, CARD32, CARD32, CARD32);
-void XDGAUnmapFramebuffer(int);
-unsigned char* XDGAGetMappedMemory(int);
+static Bool
+XDGAMapFramebuffer(int, char *, mmapOffset, CARD32, CARD32, CARD32);
+static void XDGAUnmapFramebuffer(int);
+static unsigned char* XDGAGetMappedMemory(int);
 
 #define XDGACheckExtension(dpy,i,val) \
   XextCheckExtension (dpy, i, xdga_extension_name, val)
@@ -218,11 +227,12 @@ Bool XDGAOpenFramebuffer(
     Display	*dpy,
     int 	screen
 ){
+    mmapOffset offset;
     XExtDisplayInfo *info = xdga_find_display (dpy);
     xXDGAOpenFramebufferReply rep;
     xXDGAOpenFramebufferReq *req;
     char *deviceName = NULL;
-    Bool ret;
+    Bool ret = True;
 
     XDGACheckExtension (dpy, info, False);
 
@@ -242,9 +252,20 @@ Bool XDGAOpenFramebuffer(
 	_XRead(dpy, deviceName, rep.length << 2);
     }
 
-    ret = XDGAMapFramebuffer(screen, deviceName,
-				(unsigned char*)(long)rep.mem1, 
-				rep.size, rep.offset, rep.extra);
+    if ((offset = rep.mem2)) {
+	if (sizeof(offset) == sizeof(rep.mem2))
+	    ret = False;
+	else
+	    /* Not using mmapOffset here should avoid GCC noise */
+	    offset = (unsigned long long)offset << 32;
+    }
+
+    if (ret == True) {
+	offset |= rep.mem1;
+
+	ret = XDGAMapFramebuffer(screen, deviceName, offset,
+				 rep.size, rep.offset, rep.extra);
+    }
 
     if(deviceName)
 	Xfree(deviceName);	
@@ -742,7 +763,7 @@ void XDGAKeyEventToXKeyEvent(
 #include <signal.h>
 #include <unistd.h>
 
-#if defined(SVR4) && !defined(sun) && !defined(SCO325)
+#if defined(SVR4) && !defined(sun) && !defined(__SCO__)
 #define DEV_MEM "/dev/pmem"
 #elif defined(SVR4) && defined(sun)
 #define DEV_MEM "/dev/xsvc"
@@ -755,7 +776,7 @@ void XDGAKeyEventToXKeyEvent(
 
 
 typedef struct _DGAMapRec{
-  unsigned char *physical;
+  mmapOffset physical;
   unsigned char *virtual;
   CARD32 size;
   int fd;
@@ -764,13 +785,13 @@ typedef struct _DGAMapRec{
 } DGAMapRec, *DGAMapPtr;
 
 static Bool
-DGAMapPhysical(int, char*, unsigned char*, CARD32, CARD32, CARD32, DGAMapPtr); 
+DGAMapPhysical(int, char*, mmapOffset, CARD32, CARD32, CARD32, DGAMapPtr); 
 static void DGAUnmapPhysical(DGAMapPtr);
 
 static DGAMapPtr _Maps = NULL;
 
 
-unsigned char*
+static unsigned char*
 XDGAGetMappedMemory(int screen)
 {
     DGAMapPtr pMap = _Maps;
@@ -787,11 +808,11 @@ XDGAGetMappedMemory(int screen)
     return pntr;
 }
 
-Bool
+static Bool
 XDGAMapFramebuffer(
    int screen,
    char *name,			/* optional device name */
-   unsigned char* base,		/* physical memory */
+   mmapOffset base,		/* physical memory address */
    CARD32 size,			/* size */
    CARD32 offset,		/* optional offset */
    CARD32 extra			/* optional extra data */
@@ -806,11 +827,17 @@ XDGAMapFramebuffer(
      pMap = pMap->next;
    }
 
+#if 0
    if(extra & XDGANeedRoot) {
     /* we should probably check if we have root permissions and
        return False here */
 
+    /*
+     * On the other hand, checking for root permission is pointless, given that
+     * DGAMapPhysical() returns False on EACCESS anyway.
+     */
    }
+#endif
 
    pMap = (DGAMapPtr)Xmalloc(sizeof(DGAMapRec));
 
@@ -825,7 +852,7 @@ XDGAMapFramebuffer(
    return result;
 }
 
-void
+static void
 XDGAUnmapFramebuffer(int screen)
 {
    DGAMapPtr pMap = _Maps;
@@ -857,7 +884,7 @@ static Bool
 DGAMapPhysical(
    int screen,
    char *name,			/* optional device name */
-   unsigned char* base,		/* physical memory */
+   mmapOffset base,		/* physical memory address */
    CARD32 size,			/* size */
    CARD32 offset,		/* optional offset */
    CARD32 extra,		/* optional extra data */
@@ -927,10 +954,31 @@ DGAMapPhysical(
 #ifndef MAP_FILE
 #define MAP_FILE 0
 #endif
+#ifdef sun
+    if (!name) {
+	if (((pMap->fd = open(DEV_MEM, O_RDWR)) < 0) &&
+	    ((pMap->fd = open("/dev/fbs/aperture", O_RDWR)) < 0))
+	    return False;
+    } else
+#else
     if (!name)
-	    name = DEV_MEM;
+	name = DEV_MEM;
+#endif
     if ((pMap->fd = open(name, O_RDWR)) < 0)
 	return False;
+#ifdef linux
+    /*
+     * If we are to mmap() something in /proc/bus/pci, ensure we mmap() PCI
+     * memory.  Oddly enough the default seems to be PCI I/O for some kernels.
+     * Also, avoid having to #include <linux/pci.h> here.
+     */
+# ifndef PCIIOC_MMAP_IS_MEM
+#  define PCIIOC_MMAP_IS_MEM \
+	  (('P' << 24) | ('C' << 16) | ('I' << 8) | 0x02)
+# endif
+    if (!memcmp(name, "/proc/bus/pci/", 14))
+	ioctl(pMap->fd, PCIIOC_MMAP_IS_MEM, 0);	/* Ignore errors */
+#endif
     pMap->virtual = mmap(NULL, size, PROT_READ | PROT_WRITE, 
 			MAP_FILE | MAP_SHARED, pMap->fd, (off_t)base);
     if (pMap->virtual == (void *)-1)
