@@ -1,31 +1,34 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis6326_video.c,v 1.14 2003/11/11 18:04:30 twini Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis6326_video.c,v 1.21 2004/06/21 00:43:22 twini Exp $ */
+/* $XdotOrg$ */
 /*
  * Xv driver for SiS 5597/5598, 6236 and 530/620.
  *
- * Copyright 2002, 2003 by Thomas Winischhofer, Vienna, Austria.
+ * Copyright (C) 2001-2004 by Thomas Winischhofer, Vienna, Austria.
  *
- * Based on sis_video.c which is
- *    Copyright 2002, 2003 by Thomas Winischhofer, Vienna, Austria.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1) Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2) Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3) The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holder not be used in
- * advertising or publicity pertaining to distribution of the software without
- * specific, written prior permission.  The copyright holder makes no representations
- * about the suitability of this software for any purpose.  It is provided
- * "as is" without express or implied warranty.
- *
- * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * Author:	Thomas Winischhofer <thomas@winischhofer.net>
+ *
  */
 
 #include "sis.h"
@@ -43,7 +46,6 @@
 #include "xf86xv.h"
 #include "Xv.h"
 #include "xaa.h"
-#include "xaalocal.h"
 #include "dixstruct.h"
 #include "fourcc.h"
 
@@ -526,6 +528,13 @@ SIS6326SetupImageVideo(ScreenPtr pScreen)
     XF86VideoAdaptorPtr adapt;
     SISPortPrivPtr pPriv;
 
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,1,99,1,0)
+    XAAInfoRecPtr pXAA = pSiS->AccelInfoPtr;
+
+    if (!pXAA || !pXAA->FillSolidRects)
+	return NULL;
+#endif
+
     if(!(adapt = xcalloc(1, sizeof(XF86VideoAdaptorRec) +
                             sizeof(SISPortPrivRec) +
                             sizeof(DevUnion))))
@@ -597,7 +606,7 @@ SIS6326SetupImageVideo(ScreenPtr pScreen)
     return adapt;
 }
 
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,0,0)
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,3,0)
 static Bool
 RegionsEqual(RegionPtr A, RegionPtr B)
 {
@@ -1266,17 +1275,17 @@ SIS6326StopVideo(ScrnInfoPtr pScrn, pointer data, Bool shutdown)
 
   if(shutdown) {
      if(pPriv->videoStatus & CLIENT_VIDEO_ON) {
-       close_overlay(pSiS, pPriv);
-       pPriv->mustwait = 1;
+        close_overlay(pSiS, pPriv);
+        pPriv->mustwait = 1;
      }
      SIS6326FreeOverlayMemory(pScrn);
      pPriv->videoStatus = 0;
      pSiS->VideoTimerCallback = NULL;
   } else {
      if(pPriv->videoStatus & CLIENT_VIDEO_ON) {
-       pPriv->videoStatus = OFF_TIMER | CLIENT_VIDEO_ON;
-       pPriv->offTime = currentTime.milliseconds + OFF_DELAY;
-       pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+        pPriv->videoStatus = OFF_TIMER | CLIENT_VIDEO_ON;
+        pPriv->offTime = currentTime.milliseconds + OFF_DELAY;
+        pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
      }
   }
 }
@@ -1315,7 +1324,7 @@ SIS6326PutImage(
    pPriv->height = height;
    pPriv->width = width;
 
-   /* TW: Pixel formats:
+   /* Pixel formats:
       1. YU12:  3 planes:       H    V
                Y sample period  1    1   (8 bit per pixel)
 	       V sample period  2    2	 (8 bit per pixel, subsampled)
@@ -1390,7 +1399,7 @@ SIS6326PutImage(
    /* update cliplist */
    if(  pPriv->autopaintColorKey &&
         (pPriv->grabbedByV4L ||
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,0,0)
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,3,0)
 	 !RegionsEqual(&pPriv->clip, clipBoxes)) ) {
 #else
          !REGION_EQUAL(pScrn->pScreen, &pPriv->clip, clipBoxes)) ) {
@@ -1399,8 +1408,8 @@ SIS6326PutImage(
      if(!pPriv->grabbedByV4L)
      	REGION_COPY(pScrn->pScreen, &pPriv->clip, clipBoxes);
      /* draw these */
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,2,99,0,0)
-     XAAFillSolidRects(pScrn, pPriv->colorKey, GXcopy, ~0,
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,1,99,1,0)
+     (*pSiS->AccelInfoPtr->FillSolidRects)(pScrn, pPriv->colorKey, GXcopy, ~0,
                     REGION_NUM_RECTS(clipBoxes),
                     REGION_RECTS(clipBoxes));
 #else
@@ -1488,33 +1497,32 @@ SIS6326VideoTimerCallback (ScrnInfoPtr pScrn, Time now)
 
     if(pSiS->adaptor) {
     	pPriv = GET_PORT_PRIVATE(pScrn);
-	if(!pPriv->videoStatus)
-	   pPriv = NULL;
+	if(!pPriv->videoStatus) pPriv = NULL;
     }
 
     if(pPriv) {
-      if(pPriv->videoStatus & TIMER_MASK) {
-        UpdateCurrentTime();
-	if(pPriv->offTime < currentTime.milliseconds) {
+       if(pPriv->videoStatus & TIMER_MASK) {
           if(pPriv->videoStatus & OFF_TIMER) {
-              /* Turn off the overlay */
-	      sridx = inSISREG(SISSR); cridx = inSISREG(SISCR);
-              close_overlay(pSiS, pPriv);
-	      outSISREG(SISSR, sridx); outSISREG(SISCR, cridx);
-	      pPriv->mustwait = 1;
-              pPriv->videoStatus = FREE_TIMER;
-              pPriv->freeTime = currentTime.milliseconds + FREE_DELAY;
-	      pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+	     if(pPriv->offTime < now) {
+                /* Turn off the overlay */
+	        sridx = inSISREG(SISSR); cridx = inSISREG(SISCR);
+                close_overlay(pSiS, pPriv);
+	        outSISREG(SISSR, sridx); outSISREG(SISCR, cridx);
+	        pPriv->mustwait = 1;
+                pPriv->videoStatus = FREE_TIMER;
+                pPriv->freeTime = now + FREE_DELAY;
+	        pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+	     }
+          } else if(pPriv->videoStatus & FREE_TIMER) {  
+             if(pPriv->freeTime < now) {
+                SIS6326FreeOverlayMemory(pScrn);
+	        pPriv->mustwait = 1;
+                pPriv->videoStatus = 0;
+             }
           } else
-	  if(pPriv->videoStatus & FREE_TIMER) {  
-              SIS6326FreeOverlayMemory(pScrn);
-	      pPriv->mustwait = 1;
-              pPriv->videoStatus = 0;
-          }
-        } else
-	  pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
-      }
-   }
+	     pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+       }
+    }
 }
 
 /* Offscreen surface stuff for v4l */
@@ -1659,8 +1667,8 @@ SIS6326DisplaySurface (
    SIS6326DisplayVideo(pScrn, pPriv);
 
    if(pPriv->autopaintColorKey) {
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,2,99,0,0)
-   	XAAFillSolidRects(pScrn, pPriv->colorKey, GXcopy, ~0,
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,1,99,1,0)
+   	(*XAAPTR(pScrn)->FillSolidRects)(pScrn, pPriv->colorKey, GXcopy, ~0,
                     REGION_NUM_RECTS(clipBoxes),
                     REGION_RECTS(clipBoxes));
 #else
