@@ -28,7 +28,7 @@
  *	    Massimiliano Ghilardi, max@Linuz.sns.it, some fixes to the
  *				   clockchip programming code.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/trident/trident_driver.c,v 1.189 2003/11/06 18:38:09 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/trident/trident_driver.c,v 1.193 2005/02/18 02:55:10 dawes Exp $ */
 
 #include "xf1bpp.h"
 #include "xf4bpp.h"
@@ -500,6 +500,7 @@ static const char *vgahwSymbols[] = {
     "vgaHWSaveScreen",
     "vgaHWSetMmioFuncs",
     "vgaHWUnlock",
+    "vgaHWVBlankKGA",
     NULL
 };
 
@@ -1923,6 +1924,7 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	    chipset = "Blade3D";
 	    pTrident->NewClockCode = TRUE;
 	    pTrident->frequency = NTSC;
+	    pTrident->UsePCIRetry = TRUE; /* To avoid lockups */
 	    break;
 	case CYBERBLADEI7:
     	    pTrident->ddc1Read = Tridentddc1Read;
@@ -2604,6 +2606,8 @@ TRIDENTModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     TRIDENTPtr pTrident = TRIDENTPTR(pScrn);
     TRIDENTRegPtr tridentReg;
 
+    WAITFORVSYNC;
+
     TridentFindClock(pScrn,mode->Clock);
 
     switch (pTrident->Chipset) {
@@ -2657,6 +2661,7 @@ TRIDENTModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
     vgaHWUnlock(hwp);
     /* Initialise the ModeReg values */
+    hwp->Flags |= VGA_FIX_SYNC_PULSES;
     if (!vgaHWInit(pScrn, mode))
 	return FALSE;
     pScrn->vtSema = TRUE;
@@ -3182,6 +3187,9 @@ TRIDENTLeaveVT(int scrnIndex, int flags)
     TRIDENTPtr pTrident = TRIDENTPTR(pScrn);
     vgaHWPtr hwp = VGAHWPTR(pScrn);
 
+    if (!pTrident->NoAccel)
+	pTrident->AccelInfoRec->Sync(pScrn);
+
     TRIDENTRestore(pScrn);
     vgaHWLock(hwp);
 
@@ -3205,6 +3213,9 @@ TRIDENTCloseScreen(int scrnIndex, ScreenPtr pScreen)
     vgaHWPtr hwp = VGAHWPTR(pScrn);
     TRIDENTPtr pTrident = TRIDENTPTR(pScrn);
 
+    if (!pTrident->NoAccel)
+	pTrident->AccelInfoRec->Sync(pScrn);
+	
     if (xf86IsPc98())
 	PC98TRIDENTDisable(pScrn);
 
@@ -3266,14 +3277,14 @@ TRIDENTValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 		       mode->VDisplay,
 		       LCD[pTrident->lcdMode].display_x,
 		       LCD[pTrident->lcdMode].display_y);
-	    return(MODE_BAD);
+	    return(MODE_PANEL);
 	}
 	if (((float)mode->HDisplay/(float)mode->VDisplay) > 2.0) {
 	    xf86DrvMsg(scrnIndex,X_INFO, "Removing mode (%dx%d) "
 		       "unusual aspect ratio\n",
 		       mode->HDisplay,
 		       mode->VDisplay);
-	    return(MODE_BAD);
+	    return(MODE_ASPECT_RATIO);
 	}
     }
     return (MODE_OK);
