@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/via/via_swov.c,v 1.10 2003/12/17 19:01:59 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/via/via_swov.c,v 1.12 2004/12/15 01:26:50 tsi Exp $ */
 /*
  * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
  * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
@@ -32,7 +32,6 @@
 #include "via_capture.h"
 #include "via.h"
 #include "ddmpeg.h"
-#include "xf86drm.h"
 
 #include "via_overlay.h"
 #include "via_driver.h"
@@ -65,19 +64,18 @@ unsigned long VIAVidCreateSurface(ScrnInfoPtr pScrn, LPDDSURFACEDESC lpDDSurface
 {
     VIAPtr  pVia = VIAPTR(pScrn);
     unsigned long   dwWidth, dwHeight, dwPitch=0;
-    unsigned long   dwRet=PI_OK;
     unsigned long   size;
     unsigned long   dwAddr;
     unsigned long   HQVFBSIZE = 0, SWFBSIZE = 0;
     int     iCount;        /* iCount for clean HQV FB use */
     unsigned char    *lpTmpAddr;    /* for clean HQV FB use */
     VIAHWRec *hwDiff = &pVia->ViaHW;
-    
+    unsigned long retCode;
     
     DBG_DD(ErrorF("//VIAVidCreateSurface: \n"));
 
     if ( lpDDSurfaceDesc == NULL )
-        return PI_ERR;
+        return BadAccess;
         
     ErrorF("Creating %lu surface\n", lpDDSurfaceDesc->dwFourCC);
 
@@ -120,8 +118,8 @@ unsigned long VIAVidCreateSurface(ScrnInfoPtr pScrn, LPDDSURFACEDESC lpDDSurface
             SWFBSIZE = dwPitch*dwHeight;    /*YUYV*/
 
 	    VIAFreeLinear(&pVia->swov.SWOVMem);            
-            if(VIAAllocLinear(&pVia->swov.SWOVMem, pScrn, SWFBSIZE * 2))
-            	return BadAccess;
+            if(Success != (retCode = VIAAllocLinear(&pVia->swov.SWOVMem, pScrn, SWFBSIZE * 2)))
+            	return retCode;
             
             dwAddr = pVia->swov.SWOVMem.base;
             /* fill in the SW buffer with 0x8000 (YUY2-black color) to clear FB buffer*/
@@ -169,8 +167,8 @@ unsigned long VIAVidCreateSurface(ScrnInfoPtr pScrn, LPDDSURFACEDESC lpDDSurface
                 size = HQVFBSIZE*2;
             
 	    VIAFreeLinear(&pVia->swov.HQVMem);                
-            if(VIAAllocLinear(&pVia->swov.HQVMem, pScrn, size))
-            	return BadAccess;
+            if(Success != (retCode = VIAAllocLinear(&pVia->swov.HQVMem, pScrn, size)))
+            	return retCode;
             dwAddr = pVia->swov.HQVMem.base;
 /*            dwAddr = pVia->swov.SWOVlinear->offset * depth + SWOVFBSIZE; */
 
@@ -252,8 +250,8 @@ unsigned long VIAVidCreateSurface(ScrnInfoPtr pScrn, LPDDSURFACEDESC lpDDSurface
             SWFBSIZE = dwPitch * dwHeight * 1.5;    /* 1.5 bytes per pixel */
 
 	    VIAFreeLinear(&pVia->swov.SWfbMem);                
-	    if(VIAAllocLinear(&pVia->swov.SWfbMem, pScrn, 2 * SWFBSIZE))
-	    	return BadAccess;
+	    if(Success != (retCode = VIAAllocLinear(&pVia->swov.SWfbMem, pScrn, 2 * SWFBSIZE)))
+	    	return retCode;
 	    dwAddr = pVia->swov.SWfbMem.base;
 	    
 	    DEBUG(ErrorF("dwAddr for SWfbMem is %lu\n", dwAddr));
@@ -315,8 +313,8 @@ if (!(pVia->swov.gdwVideoFlagSW & SW_USE_HQV))
                 size = HQVFBSIZE * 2;
 
 	    VIAFreeLinear(&pVia->swov.HQVMem);                
-	    if(VIAAllocLinear(&pVia->swov.HQVMem, pScrn, size))
-	    	return BadAccess;
+	    if(Success != (retCode = VIAAllocLinear(&pVia->swov.HQVMem, pScrn, size)))
+	    	return retCode;
             
             dwAddr = pVia->swov.HQVMem.base;
 	    DEBUG(ErrorF("dwAddr for HQV is %lu\n", dwAddr));
@@ -358,7 +356,7 @@ if (!(pVia->swov.gdwVideoFlagSW & SW_USE_HQV))
             break;
     }
 
-    return dwRet;
+    return Success;
 
 } /*VIAVidCreateSurface*/
 
