@@ -1,5 +1,5 @@
 /*
- * $XFree86: xc/programs/Xserver/hw/xfree86/xaa/xaaPict.c,v 1.18 2003/04/23 18:35:34 eich Exp $
+ * $XFree86: xc/programs/Xserver/hw/xfree86/xaa/xaaPict.c,v 1.19 2004/01/26 18:54:55 mvojkovi Exp $
  *
  * Copyright  2000 Keith Packard, member of The XFree86 Project, Inc.
  *
@@ -219,6 +219,9 @@ XAADoComposite (
     ySrc += pSrc->pDrawable->y;
 
     if(pMask) {
+	if(pMask->componentAlpha)
+	    return FALSE;
+ 
 	/* for now we only do it if there is a 1x1 (solid) source */
 
 	if((pSrc->pDrawable->width == 1) && (pSrc->pDrawable->height == 1)) {
