@@ -1,10 +1,12 @@
+/* $XTermId: fontutils.c,v 1.135 2005/01/10 22:52:58 tom Exp $ */
+
 /*
- * $XFree86: xc/programs/xterm/fontutils.c,v 1.42 2003/10/28 23:11:58 dickey Exp $
+ * $XFree86: xc/programs/xterm/fontutils.c,v 1.49 2005/01/14 01:50:02 dickey Exp $
  */
 
 /************************************************************
 
-Copyright 1998-2002,2003 by Thomas E. Dickey
+Copyright 1998-2004,2005 by Thomas E. Dickey
 
                         All Rights Reserved
 
@@ -43,6 +45,7 @@ authorization.
 #define RES_OFFSET(field)	XtOffsetOf(SubResourceRec, field)
 
 #include <fontutils.h>
+#include <X11/Xmu/Drawing.h>
 
 #include <main.h>
 #include <data.h>
@@ -115,6 +118,10 @@ typedef struct {
     char *end;
 } FontNameProperties;
 
+#if OPT_SHIFT_FONTS
+static void lookupOneFontSize(TScreen *, int);
+#endif
+
 /*
  * Returns the fields from start to stop in a dash- separated string.  This
  * function will modify the source, putting '\0's in the appropiate place and
@@ -168,8 +175,8 @@ get_font_name_props(Display * dpy, XFontStruct * fs, char *result)
     static FontNameProperties props;
     static char *last_name;
 
-    register XFontProp *fp;
-    register int i;
+    XFontProp *fp;
+    int i;
     Atom fontatom = XInternAtom(dpy, "FONT", False);
     char *name;
     char *str;
@@ -421,7 +428,7 @@ xtermSpecialFont(unsigned atts, unsigned chrset)
  * XLFD allows '?' as a wildcard, but we do not handle that (no one seems
  * to use it).
  */
-static Boolean
+static Bool
 same_font_name(char *pattern, char *match)
 {
     while (*pattern && *match) {
@@ -437,7 +444,9 @@ same_font_name(char *pattern, char *match)
 		return False;
 	    }
 	} else {
-	    if (char2lower(*pattern++) != char2lower(*match++))
+	    int p = char2lower(*pattern++);
+	    int m = char2lower(*match++);
+	    if (p != m)
 		return False;
 	}
     }
@@ -513,7 +522,7 @@ is_double_width_font(XFontStruct * fs)
 #define is_double_width_font(fs) 0
 #endif
 
-#if OPT_WIDE_CHARS && defined(XRENDERFONT) && defined(HAVE_TYPE_FCCHAR32)
+#if OPT_WIDE_CHARS && OPT_RENDERFONT && defined(HAVE_TYPE_FCCHAR32)
 #define HALF_WIDTH_TEST_STRING "1234567890"
 
 /* '1234567890' in Chinese characters in UTF-8 */
@@ -554,8 +563,8 @@ is_double_width_font_xft(Display * dpy, XftFont * font)
     if (gi1.xOff != gi2.xOff)	/* Not a fixed-width font */
 	return 0;
 
-    XftTextExtentsUtf8(dpy, font, (FcChar8 *) hwstr, strlen(hwstr), &gi1);
-    XftTextExtentsUtf8(dpy, font, (FcChar8 *) fwstr, strlen(fwstr), &gi2);
+    XftTextExtentsUtf8(dpy, font, (FcChar8 *) hwstr, (int) strlen(hwstr), &gi1);
+    XftTextExtentsUtf8(dpy, font, (FcChar8 *) fwstr, (int) strlen(fwstr), &gi2);
 
     /*
      * fontconfig and Xft prior to 2.2(?) set the width of half-width
@@ -613,8 +622,7 @@ xtermLoadFont(TScreen * screen,
     Pixel new_revers;
     char *tmpname = NULL;
     char normal[MAX_FONTNAME];
-    Boolean proportional = False;
-    int ch;
+    Bool proportional = False;
 
     memset(&myfonts, 0, sizeof(myfonts));
     if (fonts != 0)
@@ -628,7 +636,8 @@ xtermLoadFont(TScreen * screen,
 	    return 0;
     }
 
-    TRACE(("xtermLoadFont normal %s\n", myfonts.f_n));
+    TRACE(("xtermLoadFont #%d normal %s\n", fontnum, NonNull(myfonts.f_n)));
+    TRACE(("xtermLoadFont #%d bold   %s\n", fontnum, NonNull(myfonts.f_b)));
 
     if (!(nfs = XLoadQueryFont(screen->display, myfonts.f_n)))
 	goto bad;
@@ -652,6 +661,7 @@ xtermLoadFont(TScreen * screen,
 	} else if (same_font_size(nfs, bfs)
 		   && got_bold_font(screen->display, bfs, myfonts.f_b)) {
 	    TRACE(("...got a matching bold font\n"));
+	    screen->bold_font_names[fontnum] = x_strdup(myfonts.f_b);
 	} else {
 	    XFreeFont(screen->display, bfs);
 	    bfs = nfs;
@@ -660,6 +670,8 @@ xtermLoadFont(TScreen * screen,
     } else if ((bfs = XLoadQueryFont(screen->display, myfonts.f_b)) == 0) {
 	bfs = nfs;
 	TRACE(("...cannot load bold font %s\n", myfonts.f_b));
+    } else {
+	screen->bold_font_names[fontnum] = x_strdup(myfonts.f_b);
     }
 
     /*
@@ -786,7 +798,7 @@ xtermLoadFont(TScreen * screen,
     xgcv.font = nfs->fid;
     xgcv.foreground = new_normal;
     xgcv.background = new_revers;
-    xgcv.graphics_exposures = TRUE;	/* default */
+    xgcv.graphics_exposures = True;	/* default */
     xgcv.function = GXcopy;
 
     new_normalGC = XtGetGC((Widget) term, mask, &xgcv);
@@ -859,26 +871,34 @@ xtermLoadFont(TScreen * screen,
      * characters.  Check that they are all present.  The null character
      * (0) is special, and is not used.
      */
-#ifdef XRENDERFONT
-    if (screen->renderFont != 0) {
+#if OPT_RENDERFONT
+    if (UsingRenderFont()) {
 	/*
 	 * FIXME: we shouldn't even be here if we're using Xft.
 	 */
 	screen->fnt_boxes = False;
+	TRACE(("assume Xft missing line-drawing chars\n"));
     } else
 #endif
     {
+	unsigned ch;
+
 	for (ch = 1; ch < 32; ch++) {
-	    int n = ch;
+	    unsigned n = ch;
 #if OPT_WIDE_CHARS
-	    if (screen->utf8_mode) {
+	    if (screen->utf8_mode || screen->unicode_font) {
 		n = dec2ucs(ch);
 		if (n == UCS_REPL)
 		    continue;
 	    }
 #endif
-	    if (xtermMissingChar(n, nfs)
-		|| xtermMissingChar(n, bfs)) {
+	    if (xtermMissingChar(n, nfs)) {
+		TRACE(("missing normal char #%d\n", n));
+		screen->fnt_boxes = False;
+		break;
+	    }
+	    if (xtermMissingChar(n, bfs)) {
+		TRACE(("missing bold char #%d\n", n));
 		screen->fnt_boxes = False;
 		break;
 	    }
@@ -903,7 +923,7 @@ xtermLoadFont(TScreen * screen,
 	if (fontnum == fontMenu_fontescape) {
 	    set_sensitivity(term->screen.fontMenu,
 			    fontMenuEntries[fontMenu_fontescape].widget,
-			    TRUE);
+			    True);
 	}
 #if OPT_SHIFT_FONTS
 	screen->menu_font_sizes[fontnum] = FontSize(nfs);
@@ -959,7 +979,7 @@ typedef struct {
 Bool
 xtermLoadVTFonts(XtermWidget w, char *myName, char *myClass)
 {
-    static Boolean initialized = False;
+    static Bool initialized = False;
     static SubResourceRec original, referenceRec, subresourceRec;
 
     /*
@@ -982,7 +1002,7 @@ xtermLoadVTFonts(XtermWidget w, char *myName, char *myClass)
 	Sres(XtNfont6, XtCFont6, menu_font_names[fontMenu_font6], NULL),
     };
     Cardinal n;
-    Boolean status = True;
+    Bool status = True;
 
     if (!initialized) {
 
@@ -1002,7 +1022,7 @@ xtermLoadVTFonts(XtermWidget w, char *myName, char *myClass)
 	TRACE(("xtermLoadVTFonts(%s, %s)\n", myName, myClass));
 
 	memset(&subresourceRec, 0, sizeof(subresourceRec));
-	XtGetSubresources((Widget) w, (XtPointer) & subresourceRec,
+	XtGetSubresources((Widget) w, (XtPointer) &subresourceRec,
 			  myName, myClass,
 			  font_resources,
 			  (Cardinal) XtNumber(font_resources),
@@ -1041,10 +1061,12 @@ void
 HandleLoadVTFonts(Widget w GCC_UNUSED,
 		  XEvent * event GCC_UNUSED,
 		  String * params GCC_UNUSED,
-		  Cardinal * param_count GCC_UNUSED)
+		  Cardinal *param_count GCC_UNUSED)
 {
+    static char empty[] = "";	/* appease strict compilers */
+
     char buf[80];
-    char *myName = (*param_count > 0) ? params[0] : "";
+    char *myName = (*param_count > 0) ? params[0] : empty;
     char *convert = (*param_count > 1) ? params[1] : myName;
     char *myClass = (char *) MyStackAlloc(strlen(convert), buf);
     int n;
@@ -1053,7 +1075,7 @@ HandleLoadVTFonts(Widget w GCC_UNUSED,
     strcpy(myClass, convert);
     if (*param_count == 1
 	&& islower(CharOf(myClass[0])))
-	myClass[0] = toupper(myClass[0]);
+	myClass[0] = toupper(CharOf(myClass[0]));
 
     if (xtermLoadVTFonts(term, myName, myClass)) {
 	/*
@@ -1066,7 +1088,7 @@ HandleLoadVTFonts(Widget w GCC_UNUSED,
 	    font_number = fontMenu_lastBuiltin;
 	for (n = 0; n < NMENUFONTS; ++n)
 	    term->screen.menu_font_sizes[n] = 0;
-	SetVTFont(font_number, TRUE,
+	SetVTFont(font_number, True,
 		  ((font_number == fontMenu_fontdefault)
 		   ? &(term->misc.default_font)
 		   : NULL));
@@ -1093,6 +1115,17 @@ xtermSetCursorBox(TScreen * screen)
     screen->box = VTbox;
 }
 
+#define CACHE_XFT(dst,src) if (src != 0) {\
+	    dst[fontnum] = src;\
+	    TRACE(("%s[%d] = %d (%d,%d) by %d\n",\
+		#dst,\
+	    	fontnum,\
+		src->height,\
+		src->ascent,\
+		src->descent,\
+		src->max_advance_width));\
+	}
+
 /*
  * Compute useful values for the font/window sizes
  */
@@ -1104,55 +1137,137 @@ xtermComputeFontInfo(TScreen * screen,
 {
     int i, j, width, height;
 
-#ifdef XRENDERFONT
-    Display *dpy = screen->display;
-    if (!screen->renderFont && term->misc.face_name) {
-	XftPattern *pat, *match;
-	XftResult result;
+#if OPT_RENDERFONT
+    /*
+     * xterm contains a lot of references to fonts, assuming they are fixed
+     * size.  This chunk of code overrides the actual font-selection (see
+     * drawXtermText()), if the user has selected render-font.  All of the
+     * font-loading for fixed-fonts still goes on whether or not this chunk
+     * overrides it.
+     */
+    if (term->misc.render_font && !IsIconWin(screen, win)) {
+	Display *dpy = screen->display;
+	int fontnum = screen->menu_font_number;
+	XftFont *norm = screen->renderFontNorm[fontnum];
+	XftFont *bold = screen->renderFontBold[fontnum];
+#if OPT_RENDERWIDE
+	XftFont *wnorm = screen->renderWideNorm[fontnum];
+	XftFont *wbold = screen->renderWideBold[fontnum];
+#endif
 
-	pat = XftNameParse(term->misc.face_name);
-	XftPatternBuild(pat,
-			XFT_FAMILY, XftTypeString, "mono",
-			XFT_SIZE, XftTypeInteger, term->misc.face_size,
-			XFT_SPACING, XftTypeInteger, XFT_MONO,
-			(void *) 0);
-	match = XftFontMatch(dpy, DefaultScreen(dpy), pat, &result);
-	screen->renderFont = XftFontOpenPattern(dpy, match);
-	if (!screen->renderFont && match)
-	    XftPatternDestroy(match);
-	if (screen->renderFont) {
+	if (norm == 0 && term->misc.face_name) {
+	    XftPattern *pat, *match;
+	    XftResult result;
+	    double face_size = term->misc.face_size;
+
+	    TRACE(("xtermComputeFontInfo norm(face %s, size %f)\n",
+		   term->misc.face_name,
+		   term->misc.face_size));
+
+#if OPT_SHIFT_FONTS
+	    /*
+	     * If the user is switching font-sizes, make it follow the same
+	     * ratios to the default as the fixed fonts would, for easy
+	     * comparison.  There will be some differences since the fixed
+	     * fonts have a variety of height/width ratios, but this is simpler
+	     * than adding another resource value - and as noted above, the
+	     * data for the fixed fonts are available.
+	     */
+	    lookupOneFontSize(screen, fontnum);
+	    if (fontnum != fontMenu_fontdefault) {
+		int num = screen->menu_font_sizes[fontnum];
+		int den = screen->menu_font_sizes[0];
+		face_size = (1.0 * face_size * num) / den;
+		TRACE(("scaled using %d/%d -> %f\n", num, den, face_size));
+	    }
+#endif
+
+	    pat = XftNameParse(term->misc.face_name);
 	    XftPatternBuild(pat,
-			    XFT_WEIGHT, XftTypeInteger, XFT_WEIGHT_BOLD,
-			    XFT_CHAR_WIDTH, XftTypeInteger, screen->renderFont->max_advance_width,
+			    XFT_FAMILY, XftTypeString, "mono",
+			    XFT_SIZE, XftTypeDouble, face_size,
+			    XFT_SPACING, XftTypeInteger, XFT_MONO,
 			    (void *) 0);
 	    match = XftFontMatch(dpy, DefaultScreen(dpy), pat, &result);
-	    screen->renderFontBold = XftFontOpenPattern(dpy, match);
-	    if (!screen->renderFontBold && match)
+	    norm = XftFontOpenPattern(dpy, match);
+	    if ((norm == 0) && match)
 		XftPatternDestroy(match);
+	    if (norm != 0) {
+		XftPatternBuild(pat,
+				XFT_WEIGHT, XftTypeInteger, XFT_WEIGHT_BOLD,
+				XFT_CHAR_WIDTH, XftTypeInteger, norm->max_advance_width,
+				(void *) 0);
+		match = XftFontMatch(dpy, DefaultScreen(dpy), pat, &result);
+		bold = XftFontOpenPattern(dpy, match);
+		if ((bold == 0) && match)
+		    XftPatternDestroy(match);
+
+		/*
+		 * FIXME:  just assume that the corresponding font has no
+		 * graphics characters.
+		 */
+		if (screen->fnt_boxes) {
+		    screen->fnt_boxes = False;
+		    TRACE(("Xft opened - will %suse internal line-drawing characters\n",
+			   screen->fnt_boxes ? "not " : ""));
+		}
+	    }
+
+	    if (pat)
+		XftPatternDestroy(pat);
+
+	    CACHE_XFT(screen->renderFontNorm, norm);
+	    CACHE_XFT(screen->renderFontBold, bold);
 
 	    /*
-	     * FIXME:  just assume that the corresponding font has no graphics
-	     * characters.
+	     * See xtermXftDrawString().
 	     */
-	    if (screen->fnt_boxes) {
-		screen->fnt_boxes = False;
-		TRACE(("Xft opened - will %suse internal line-drawing characters\n",
-		       screen->fnt_boxes ? "not " : ""));
+#if OPT_RENDERWIDE
+	    if (norm != 0 && wnorm == 0) {
+		char *face_name = (term->misc.face_wide_name
+				   ? term->misc.face_wide_name
+				   : term->misc.face_name);
+		int char_width = norm->max_advance_width * 2;
+
+		TRACE(("xtermComputeFontInfo wide(face %s, char_width %d)\n",
+		       face_name,
+		       char_width));
+
+		wnorm = XftFontOpen(dpy, DefaultScreen(dpy),
+				    XFT_FAMILY, XftTypeString, face_name,
+				    XFT_SIZE, XftTypeDouble, face_size,
+				    XFT_SPACING, XftTypeInteger, XFT_MONO,
+				    XFT_CHAR_WIDTH, XftTypeInteger, char_width,
+				    (void *) 0);
+
+		wbold = XftFontOpen(dpy, DefaultScreen(dpy),
+				    XFT_FAMILY, XftTypeString, face_name,
+				    XFT_SIZE, XftTypeDouble, face_size,
+				    XFT_SPACING, XftTypeInteger, XFT_MONO,
+				    XFT_CHAR_WIDTH, XftTypeInteger, char_width,
+				    XFT_WEIGHT, XftTypeInteger, XFT_WEIGHT_BOLD,
+				    (void *) 0);
+
+		CACHE_XFT(screen->renderWideNorm, wnorm);
+		CACHE_XFT(screen->renderWideBold, wbold);
 	    }
+#endif
+	}
+	if (norm == 0) {
+	    term->misc.render_font = False;
+	    update_font_renderfont();
+	} else {
+	    win->f_width = norm->max_advance_width;
+	    win->f_height = norm->height;
+	    win->f_ascent = norm->ascent;
+	    win->f_descent = norm->descent;
+	    if (win->f_height < win->f_ascent + win->f_descent)
+		win->f_height = win->f_ascent + win->f_descent;
+	    if (is_double_width_font_xft(screen->display, norm))
+		win->f_width >>= 1;
 	}
-	if (pat)
-	    XftPatternDestroy(pat);
     }
-    if (screen->renderFont) {
-	win->f_width = screen->renderFont->max_advance_width;
-	win->f_height = screen->renderFont->height;
-	win->f_ascent = screen->renderFont->ascent;
-	win->f_descent = screen->renderFont->descent;
-	if (win->f_height < win->f_ascent + win->f_descent)
-	    win->f_height = win->f_ascent + win->f_descent;
-	if (is_double_width_font_xft(screen->display, screen->renderFont))
-	    win->f_width >>= 1;
-    } else
+    if (!term->misc.render_font || IsIconWin(screen, win))
 #endif
     {
 	if (is_double_width_font(font)) {
@@ -1172,6 +1287,16 @@ xtermComputeFontInfo(TScreen * screen,
     win->fullheight = height;
     win->width = width - i;
     win->height = height - j;
+
+    TRACE(("xtermComputeFontInfo window %dx%d (full %dx%d), fontsize %dx%d (asc %d, dsc %d)\n",
+	   win->height,
+	   win->width,
+	   win->fullheight,
+	   win->fullwidth,
+	   win->f_height,
+	   win->f_width,
+	   win->f_ascent,
+	   win->f_descent));
 }
 
 /* save this information as a side-effect for double-sized characters */
@@ -1256,7 +1381,8 @@ xtermMissingChar(unsigned ch, XFontStruct * font)
 }
 
 /*
- * The grid is abitrary, enough resolution that nothing's lost in initialization.
+ * The grid is arbitrary, enough resolution that nothing's lost in
+ * initialization.
  */
 #define BOX_HIGH 60
 #define BOX_WIDE 60
@@ -1264,6 +1390,9 @@ xtermMissingChar(unsigned ch, XFontStruct * font)
 #define MID_HIGH (BOX_HIGH/2)
 #define MID_WIDE (BOX_WIDE/2)
 
+#define CHR_WIDE (9*BOX_WIDE)/10
+#define CHR_HIGH (9*BOX_HIGH)/10
+
 /*
  * ...since we'll scale the values anyway.
  */
@@ -1277,21 +1406,32 @@ xtermMissingChar(unsigned ch, XFontStruct * font)
  * line-drawing character).
  */
 void
-xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
+xtermDrawBoxChar(TScreen * screen,
+		 unsigned ch,
+		 unsigned flags,
+		 GC gc,
+		 int x,
+		 int y)
 {
     /* *INDENT-OFF* */
     static const short diamond[] =
     {
-	SEG(  MID_WIDE,	    BOX_HIGH/4, 3*BOX_WIDE/4,   MID_WIDE),
+	SEG(  MID_WIDE,	    BOX_HIGH/4, 3*BOX_WIDE/4,	MID_WIDE),
 	SEG(3*BOX_WIDE/4,   MID_WIDE,	  MID_WIDE,   3*BOX_HIGH/4),
-	SEG(  MID_WIDE,   3*BOX_HIGH/4,	  BOX_WIDE/4,   MID_HIGH),
+	SEG(  MID_WIDE,	  3*BOX_HIGH/4,	  BOX_WIDE/4,	MID_HIGH),
 	SEG(  BOX_WIDE/4,   MID_HIGH,	  MID_WIDE,	BOX_HIGH/4),
-	SEG(  MID_WIDE,	    BOX_HIGH/3, 2*BOX_WIDE/3,   MID_WIDE),
+	SEG(  MID_WIDE,	    BOX_HIGH/3, 2*BOX_WIDE/3,	MID_WIDE),
 	SEG(2*BOX_WIDE/3,   MID_WIDE,	  MID_WIDE,   2*BOX_HIGH/3),
-	SEG(  MID_WIDE,   2*BOX_HIGH/3,	  BOX_WIDE/3,   MID_HIGH),
+	SEG(  MID_WIDE,	  2*BOX_HIGH/3,	  BOX_WIDE/3,	MID_HIGH),
 	SEG(  BOX_WIDE/3,   MID_HIGH,	  MID_WIDE,	BOX_HIGH/3),
-	SEG(  BOX_WIDE/4,   MID_HIGH,	3*BOX_WIDE/4,   MID_HIGH),
-	SEG(  MID_WIDE,     BOX_HIGH/4,	  MID_WIDE,   3*BOX_HIGH/4),
+	SEG(  BOX_WIDE/4,   MID_HIGH,	3*BOX_WIDE/4,	MID_HIGH),
+	SEG(  MID_WIDE,	    BOX_HIGH/4,	  MID_WIDE,   3*BOX_HIGH/4),
+	-1
+    }, plus_or_minus[] =
+    {
+	SEG(  0,	  5*BOX_HIGH/6,	  CHR_WIDE,   5*BOX_HIGH/6),
+	SEG(  MID_WIDE,	  2*BOX_HIGH/6,	  MID_WIDE,   4*BOX_HIGH/6),
+	SEG(  0,	  3*BOX_HIGH/6,	  CHR_WIDE,   3*BOX_HIGH/6),
 	-1
     }, degrees[] =
     {
@@ -1339,7 +1479,7 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
 	-1
     }, scan_line_9[] =
     {
-	SEG(  0,	    3*BOX_HIGH/4, BOX_WIDE, 3 * BOX_HIGH / 4),
+	SEG(  0,	  3*BOX_HIGH/4,	  BOX_WIDE,   3*BOX_HIGH/4),
 	-1
     }, horizontal_line[] =
     {
@@ -1352,7 +1492,7 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
 	-1
     }, right_tee[] =
     {
-	SEG(  MID_WIDE,	    0, MID_WIDE,		BOX_HIGH),
+	SEG(  MID_WIDE,	    0,		  MID_WIDE,	BOX_HIGH),
 	SEG(  MID_WIDE,	    MID_HIGH,	  0,		MID_HIGH),
 	-1
     }, bottom_tee[] =
@@ -1371,32 +1511,51 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
 	-1
     }, less_than_or_equal[] =
     {
-	SEG(5*BOX_WIDE/6,   BOX_HIGH/6,	  BOX_WIDE/5,	MID_HIGH),
-	SEG(5*BOX_WIDE/6, 5*BOX_HIGH/6,	  BOX_WIDE/5,	MID_HIGH),
-	SEG(  BOX_WIDE/6, 5*BOX_HIGH/6, 5*BOX_WIDE/6, 5*BOX_HIGH/6),
+	SEG(  CHR_WIDE,	    BOX_HIGH/3,	  0,		MID_HIGH),
+	SEG(  CHR_WIDE,	  2*BOX_HIGH/3,	  0,		MID_HIGH),
+	SEG(  0,	  3*BOX_HIGH/4,	  CHR_WIDE,   3*BOX_HIGH/4),
 	-1
     }, greater_than_or_equal[] =
     {
-	SEG(  BOX_WIDE/6,  BOX_HIGH /6, 5*BOX_WIDE/6,   MID_HIGH),
-	SEG(  BOX_WIDE/6, 5*BOX_HIGH/6, 5*BOX_WIDE/6,   MID_HIGH),
-	SEG(  BOX_WIDE/6, 5*BOX_HIGH/6, 5*BOX_WIDE/6, 5*BOX_HIGH/6),
+	SEG(  0,	    BOX_HIGH/3,	  CHR_WIDE,	MID_HIGH),
+	SEG(  0,	  2*BOX_HIGH/3,	  CHR_WIDE,	MID_HIGH),
+	SEG(  0,	  3*BOX_HIGH/4,	  CHR_WIDE,   3*BOX_HIGH/4),
+	-1
+    }, greek_pi[] =
+    {
+	SEG(  0,	    MID_HIGH,	  CHR_WIDE,	MID_HIGH),
+	SEG(5*CHR_WIDE/6,   MID_HIGH,	5*CHR_WIDE/6,	CHR_HIGH),
+	SEG(2*CHR_WIDE/6,   MID_HIGH,	2*CHR_WIDE/6,	CHR_HIGH),
+	-1
+    }, not_equal_to[] =
+    {
+	SEG(2*BOX_WIDE/3, 1*BOX_HIGH/3, 1*BOX_WIDE/3,	CHR_HIGH),
+	SEG(  0,	  2*BOX_HIGH/3,	  CHR_WIDE,   2*BOX_HIGH/3),
+	SEG(  0,	    MID_HIGH,	  CHR_WIDE,	MID_HIGH),
+	-1
+    }, bullet[] =
+    {
+	SEG(  MID_WIDE,	    BOX_HIGH/4, 5*BOX_WIDE/8,	BOX_HIGH/4),
+	SEG(5*BOX_WIDE/8,   BOX_HIGH/4, 5*BOX_WIDE/8,	BOX_HIGH/3),
+	SEG(5*BOX_WIDE/8,   BOX_HIGH/3,	  MID_WIDE,	BOX_HIGH/3),
+	SEG(  MID_WIDE,	    BOX_HIGH/3,	  MID_WIDE,	BOX_HIGH/4),
 	-1
     };
     /* *INDENT-ON* */
 
     static const short *lines[] =
     {
-	0,			/* 00 */
+	0,			/* 00 (unused) */
 	diamond,		/* 01 */
-	0,			/* 02 */
-	0,			/* 03 */
-	0,			/* 04 */
-	0,			/* 05 */
-	0,			/* 06 */
+	0,			/* 02 box */
+	0,			/* 03 HT */
+	0,			/* 04 FF */
+	0,			/* 05 CR */
+	0,			/* 06 LF */
 	degrees,		/* 07 */
-	0,			/* 08 */
-	0,			/* 09 */
-	0,			/* 0A */
+	plus_or_minus,		/* 08 */
+	0,			/* 09 NL */
+	0,			/* 0A VT */
 	lower_right_corner,	/* 0B */
 	upper_right_corner,	/* 0C */
 	upper_left_corner,	/* 0D */
@@ -1414,17 +1573,18 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
 	vertical_line,		/* 19 */
 	less_than_or_equal,	/* 1A */
 	greater_than_or_equal,	/* 1B */
-	0,			/* 1C */
-	0,			/* 1D */
-	0,			/* 1E */
-	0,			/* 1F */
+	greek_pi,		/* 1C */
+	not_equal_to,		/* 1D */
+	0,			/* 1E LB */
+	bullet,			/* 1F */
     };
 
     XGCValues values;
+    unsigned long mask;
     GC gc2;
     const short *p;
-    int font_width = ((flags & DOUBLEWFONT) ? 2 : 1) * screen->fnt_wide;
-    int font_height = ((flags & DOUBLEHFONT) ? 2 : 1) * screen->fnt_high;
+    unsigned font_width = ((flags & DOUBLEWFONT) ? 2 : 1) * screen->fnt_wide;
+    unsigned font_height = ((flags & DOUBLEHFONT) ? 2 : 1) * screen->fnt_high;
 
 #if OPT_WIDE_CHARS
     /*
@@ -1432,8 +1592,8 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
      * mode, but have gotten an old-style font.
      */
     if (screen->utf8_mode
-#ifdef XRENDERFONT
-	&& screen->renderFont == 0
+#if OPT_RENDERFONT
+	&& !UsingRenderFont()
 #endif
 	&& (ch > 127)
 	&& (ch != UCS_REPL)) {
@@ -1453,38 +1613,56 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
 
     TRACE(("DRAW_BOX(%d) cell %dx%d at %d,%d%s\n",
 	   ch, font_height, font_width, y, x,
-	   ((ch < 0 || ch >= (int) (sizeof(lines) / sizeof(lines[0])))
+	   (ch >= (sizeof(lines) / sizeof(lines[0]))
 	    ? "-BAD"
 	    : "")));
 
     if (!XGetGCValues(screen->display, gc, GCBackground, &values))
 	return;
 
-    values.foreground = values.background;
-    gc2 = XCreateGC(screen->display, VWindow(screen), GCForeground, &values);
+    mask = GCForeground;
+    if (ch == 2) {
+	values.tile = XmuCreateStippledPixmap(XtScreen((Widget) term),
+					      T_COLOR(screen, TEXT_FG),
+					      T_COLOR(screen, TEXT_BG),
+					      term->core.depth);
+	if (values.stipple != XtUnspecifiedPixmap) {
+	    mask |= GCBackground | GCTile | GCFillStyle;
+	    values.fill_style = FillTiled;
+	} else {
+	    ch = (unsigned) (~0);	/* make this not match anything */
+	}
+    } else {
+	values.foreground = values.background;
+    }
+    gc2 = XCreateGC(screen->display,
+		    VWindow(screen),
+		    mask,
+		    &values);
 
-    if (!(flags & NOBACKGROUND))
-	XFillRectangle(
-			  screen->display, VWindow(screen), gc2, x, y,
-			  font_width,
-			  font_height);
+    if (!(flags & NOBACKGROUND)) {
+	XFillRectangle(screen->display, VWindow(screen), gc2, x, y,
+		       font_width,
+		       font_height);
+    }
 
     XCopyGC(screen->display, gc, (1 << GCLastBit) - 1, gc2);
     XSetLineAttributes(screen->display, gc2,
 		       (flags & BOLD)
-		       ? ((font_height > 6)
-			  ? font_height / 6
+		       ? ((font_height > 12)
+			  ? font_height / 12
 			  : 1)
-		       : ((font_height > 8)
-			  ? font_height / 8
+		       : ((font_height > 16)
+			  ? font_height / 16
 			  : 1),
 		       LineSolid,
 		       CapProjecting,
 		       JoinMiter);
 
-    if (ch >= 0
-	&& ch < (int) (sizeof(lines) / sizeof(lines[0]))
-	&& (p = lines[ch]) != 0) {
+    if (ch == 2) {
+	XmuReleaseStippledPixmap(XtScreen((Widget) term), values.tile);
+    } else if (ch < (sizeof(lines) / sizeof(lines[0]))
+	       && (p = lines[ch]) != 0) {
 	int coord[4];
 	int n = 0;
 	while (*p >= 0) {
@@ -1494,34 +1672,89 @@ xtermDrawBoxChar(TScreen * screen, int ch, unsigned flags, GC gc, int x, int y)
 		SCALE_Y(coord[1]);
 		SCALE_X(coord[2]);
 		SCALE_Y(coord[3]);
-		XDrawLine(
-			     screen->display,
-			     VWindow(screen), gc2,
-			     x + coord[0], y + coord[1],
-			     x + coord[2], y + coord[3]);
+		XDrawLine(screen->display,
+			  VWindow(screen), gc2,
+			  x + coord[0], y + coord[1],
+			  x + coord[2], y + coord[3]);
 		n = 0;
 	    }
 	}
+    } else if (screen->force_all_chars) {
+	/* bounding rectangle, for debugging */
+	XDrawRectangle(screen->display, VWindow(screen), gc2, x, y,
+		       font_width - 1,
+		       font_height - 1);
     }
-#if 0				/* bounding rectangle, for debugging */
-    else {
-	XDrawRectangle(
-			  screen->display, VWindow(screen), gc, x, y,
-			  font_width - 1,
-			  font_height - 1);
-    }
-#endif
 
     XFreeGC(screen->display, gc2);
 }
-#endif
+
+#if OPT_RENDERFONT && OPT_WIDE_CHARS
+
+/*
+ * Check if the given character has a glyph known to Xft.
+ *
+ * see xc/lib/Xft/xftglyphs.c
+ */
+Bool
+xtermXftMissing(XftFont * font, unsigned wc)
+{
+    unsigned check = XftCharIndex(term->screen.display, font, wc);
+    Bool result = False;
+
+    if (check == 0) {
+	TRACE(("missingXft %d (%d)\n", wc, ucs2dec(wc)));
+	result = True;
+    }
+    return result;
+}
+
+/*
+ * Check if the character corresponds to one of xterm's internal codes for
+ * line-drawing characters.  That is only a subset of the 1-31 codes used for
+ * graphic characters.  We want to know specifically about the line-drawing
+ * characters because the fonts used by Xft do not always give useful glyphs
+ * for line-drawing, and there is no reliable way to detect this.
+ */
+Bool
+xtermIsLineDrawing(unsigned wc)
+{
+    Bool result;
+    switch (wc) {
+    case 0x0B:			/* lower_right_corner   */
+    case 0x0C:			/* upper_right_corner   */
+    case 0x0D:			/* upper_left_corner    */
+    case 0x0E:			/* lower_left_corner    */
+    case 0x0F:			/* cross                */
+    case 0x10:			/* scan_line_1          */
+    case 0x11:			/* scan_line_3          */
+    case 0x12:			/* scan_line_7          */
+    case 0x13:			/* scan_line_9          */
+    case 0x14:			/* horizontal_line      */
+    case 0x15:			/* left_tee             */
+    case 0x16:			/* right_tee            */
+    case 0x17:			/* bottom_tee           */
+    case 0x18:			/* top_tee              */
+    case 0x19:			/* vertical_line        */
+	result = True;
+	TRACE(("xtermIsLineDrawing %d\n", wc));
+	break;
+    default:
+	result = False;
+	break;
+    }
+    return result;
+}
+#endif /* OPT_RENDERFONT && OPT_WIDE_CHARS */
+
+#endif /* OPT_BOX_CHARS */
 
 #if OPT_WIDE_CHARS
 #define MY_UCS(ucs,dec) case ucs: result = dec; break
-int
-ucs2dec(int ch)
+unsigned
+ucs2dec(unsigned ch)
 {
-    int result = ch;
+    unsigned result = ch;
     if ((ch > 127)
 	&& (ch != UCS_REPL)) {
 	switch (ch) {
@@ -1565,10 +1798,10 @@ ucs2dec(int ch)
 #undef  MY_UCS
 #define MY_UCS(ucs,dec) case dec: result = ucs; break
 
-int
-dec2ucs(int ch)
+unsigned
+dec2ucs(unsigned ch)
 {
-    int result = ch;
+    unsigned result = ch;
     if (ch < 32) {
 	switch (ch) {
 	    MY_UCS(0x25ae, 0);	/* black vertical rectangle                   */
@@ -1627,6 +1860,21 @@ xtermFindFont(TScreen * screen, int fontnum)
     return nfs;
 }
 
+static void
+lookupOneFontSize(TScreen * screen, int fontnum)
+{
+    if (screen->menu_font_sizes[fontnum] == 0) {
+	XFontStruct *fs = xtermFindFont(screen, fontnum);
+	screen->menu_font_sizes[fontnum] = -1;
+	if (fs != 0) {
+	    screen->menu_font_sizes[fontnum] = FontSize(fs);
+	    TRACE(("menu_font_sizes[%d] = %ld\n", fontnum,
+		   screen->menu_font_sizes[fontnum]));
+	    XFreeFont(screen->display, fs);
+	}
+    }
+}
+
 /*
  * Cache the font-sizes so subsequent larger/smaller font actions will go fast.
  */
@@ -1636,16 +1884,7 @@ lookupFontSizes(TScreen * screen)
     int n;
 
     for (n = 0; n < NMENUFONTS; n++) {
-	if (screen->menu_font_sizes[n] == 0) {
-	    XFontStruct *fs = xtermFindFont(screen, n);
-	    screen->menu_font_sizes[n] = -1;
-	    if (fs != 0) {
-		screen->menu_font_sizes[n] = FontSize(fs);
-		TRACE(("menu_font_sizes[%d] = %ld\n", n,
-		       screen->menu_font_sizes[n]));
-		XFreeFont(screen->display, fs);
-	    }
-	}
+	lookupOneFontSize(screen, n);
     }
 }
 
@@ -1682,7 +1921,7 @@ lookupRelativeFontSize(TScreen * screen, int old, int relative)
 	    if (relative > 1)
 		m = lookupRelativeFontSize(screen, m, relative - 1);
 	    else if (relative < -1)
-		m = lookupRelativeFontSize(screen, m, relative = 1);
+		m = lookupRelativeFontSize(screen, m, relative + 1);
 	}
     }
     return m;
@@ -1693,7 +1932,7 @@ void
 HandleLargerFont(Widget w GCC_UNUSED,
 		 XEvent * event GCC_UNUSED,
 		 String * params GCC_UNUSED,
-		 Cardinal * param_count GCC_UNUSED)
+		 Cardinal *param_count GCC_UNUSED)
 {
     if (term->misc.shift_fonts) {
 	TScreen *screen = &term->screen;
@@ -1701,7 +1940,7 @@ HandleLargerFont(Widget w GCC_UNUSED,
 
 	m = lookupRelativeFontSize(screen, screen->menu_font_number, 1);
 	if (m >= 0) {
-	    SetVTFont(m, TRUE, NULL);
+	    SetVTFont(m, True, NULL);
 	} else {
 	    Bell(XkbBI_MinorError, 0);
 	}
@@ -1713,7 +1952,7 @@ void
 HandleSmallerFont(Widget w GCC_UNUSED,
 		  XEvent * event GCC_UNUSED,
 		  String * params GCC_UNUSED,
-		  Cardinal * param_count GCC_UNUSED)
+		  Cardinal *param_count GCC_UNUSED)
 {
     if (term->misc.shift_fonts) {
 	TScreen *screen = &term->screen;
@@ -1721,7 +1960,7 @@ HandleSmallerFont(Widget w GCC_UNUSED,
 
 	m = lookupRelativeFontSize(screen, screen->menu_font_number, -1);
 	if (m >= 0) {
-	    SetVTFont(m, TRUE, NULL);
+	    SetVTFont(m, True, NULL);
 	} else {
 	    Bell(XkbBI_MinorError, 0);
 	}
@@ -1734,7 +1973,7 @@ void
 HandleSetFont(Widget w GCC_UNUSED,
 	      XEvent * event GCC_UNUSED,
 	      String * params,
-	      Cardinal * param_count)
+	      Cardinal *param_count)
 {
     int fontnum;
     VTFontNames fonts;
@@ -1837,6 +2076,8 @@ SetVTFont(int i,
 	} else {
 	    if (myfonts.f_n == 0)
 		myfonts.f_n = screen->menu_font_names[i];
+	    if (myfonts.f_b == 0)
+		myfonts.f_b = screen->bold_font_names[i];
 	    if (xtermLoadFont(screen,
 			      &myfonts,
 			      doresize, i)) {
